@@ -20,26 +20,31 @@
 #ifndef OPM_ECLIPSE_STATE_HPP
 #define OPM_ECLIPSE_STATE_HPP
 
-#include <cstddef>
-#include <memory>
-#include <vector>
-
 #include <opm/input/eclipse/EclipseState/Aquifer/AquiferConfig.hpp>
+#include <opm/input/eclipse/EclipseState/Compositional/CompositionalConfig.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseConfig.hpp>
-#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
-#include <opm/input/eclipse/EclipseState/MICPpara.hpp>
-#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FIPRegionStatistics.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FaultCollection.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/LgrCollection.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/NNC.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/TransMult.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/LgrCollection.hpp>
+#include <opm/input/eclipse/EclipseState/MICPpara.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 #include <opm/input/eclipse/EclipseState/SimulationConfig/SimulationConfig.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
+#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
+#include <opm/input/eclipse/EclipseState/Co2StoreConfig.hpp>
+#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
+
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
+#include <cstddef>
+#include <memory>
+#include <optional>
+#include <vector>
+
 namespace Opm {
     class Deck;
     class DeckKeyword;
@@ -50,6 +55,7 @@ namespace Opm {
 
 namespace Opm { namespace RestartIO {
     class RstAquifer;
+    class RstNetwork;
 }} // namespace Opm::RestartIO
 
 namespace Opm {
@@ -91,6 +97,10 @@ namespace Opm {
         virtual const FieldPropsManager& fieldProps() const;
         // Always the non-parallel field properties
         virtual const FieldPropsManager& globalFieldProps() const;
+
+        virtual void computeFipRegionStatistics();
+        const FIPRegionStatistics& fipRegionStatistics() const;
+
         const TableManager& getTableManager() const;
         const EclipseConfig& getEclipseConfig() const;
         const EclipseConfig& cfg() const;
@@ -114,6 +124,7 @@ namespace Opm {
         const TracerConfig& tracer() const;
         const MICPpara& getMICPpara() const;
         const WagHysteresisConfig& getWagHysteresis() const;
+        const Co2StoreConfig& getCo2StoreConfig() const;
 
         void reset_actnum(const std::vector<int>& new_actnum);
         void pruneDeactivatedAquiferConnections(const std::vector<std::size_t>& deactivated_cells);
@@ -124,6 +135,9 @@ namespace Opm {
         // When we know and decide to handle the same for AQUFETP and AQUCT, this part will be refactored
         void appendAqufluxSchedule(const std::unordered_set<int>& ids);
 
+        void loadRestartNetworkPressures(const RestartIO::RstNetwork& network);
+        const std::optional<std::map<std::string, double> >& getRestartNetworkPressures() const { return this->m_restart_network_pressures; }
+
         template<class Serializer>
         void serializeOp(Serializer& serializer)
         {
@@ -138,17 +152,19 @@ namespace Opm {
             serializer(m_lgrs);
             serializer(m_simulationConfig);
             serializer(aquifer_config);
+            serializer(compositional_config),
             serializer(m_transMult);
             serializer(m_faults);
             serializer(m_title);
             serializer(tracer_config);
             serializer(m_micppara);
             serializer(wag_hyst_config);
+            serializer(co2_store_config);
+            serializer(this->fipRegionStatistics_);
         }
 
         static bool rst_cmp(const EclipseState& full_state, const EclipseState& rst_state);
 
-
     private:
         void initIOConfigPostSchedule(const Deck& deck);
         void assignRunTitle(const Deck& deck);
@@ -176,13 +192,19 @@ namespace Opm {
         LgrCollection m_lgrs;
         SimulationConfig m_simulationConfig;
         AquiferConfig aquifer_config;
+        CompositionalConfig compositional_config;
         TransMult m_transMult;
         TracerConfig tracer_config;
         MICPpara m_micppara;
         WagHysteresisConfig wag_hyst_config;
+        Co2StoreConfig co2_store_config;
 
         std::string m_title{};
         FaultCollection m_faults{};
+
+        std::optional<std::map<std::string, double> > m_restart_network_pressures{std::nullopt};
+
+        std::optional<FIPRegionStatistics> fipRegionStatistics_{std::nullopt};
     };
 } // namespace Opm
 
