@@ -80,6 +80,8 @@ namespace Opm {
         explicit EclipseGrid(const Deck& deck, const int * actnum = nullptr);
 
         static bool hasGDFILE(const Deck& deck);
+        static bool hasRadialKeywords(const Deck& deck);
+        static bool hasSpiderKeywords(const Deck& deck);
         static bool hasCylindricalKeywords(const Deck& deck);
         static bool hasCornerPointKeywords(const Deck&);
         static bool hasCartesianKeywords(const Deck&);
@@ -210,6 +212,9 @@ namespace Opm {
         void resetACTNUM();
         void resetACTNUM( const std::vector<int>& actnum);
 
+        /// \brief Sets MINPVV if MINPV and MINPORV are not used
+        void setMINPVV(const std::vector<double>& minpvv);
+
         bool equal(const EclipseGrid& other) const;
         static bool hasDVDEPTHZKeywords(const Deck&);
 
