@@ -21,6 +21,7 @@
 
 #include <memory>
 #include <string>
+#include <string_view>
 #include <unordered_map>
 #include <vector>
 
@@ -49,6 +50,7 @@ public:
     virtual ~FieldPropsManager() = default;
 
     virtual void reset_actnum(const std::vector<int>& actnum);
+    void deleteMINPVV();
     const std::string& default_region() const;
     virtual std::vector<int> actnum() const;
     virtual std::vector<double> porv(bool global = false) const;
@@ -171,6 +173,8 @@ public:
     template <typename T>
     std::vector<std::string> keys() const;
 
+    virtual std::vector<std::string> fip_regions() const;
+
     const Fieldprops::FieldData<int>&
     get_int_field_data(const std::string& keyword) const;
 
