@@ -20,24 +20,40 @@
 #define OPM_RUNSPEC_HPP
 
 #include <opm/common/OpmLog/KeywordLocation.hpp>
-#include <opm/input/eclipse/EclipseState/Phase.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/Tabdims.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/Regdims.hpp>
-#include <opm/input/eclipse/EclipseState/EndpointScaling.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQParams.hpp>
-#include <opm/input/eclipse/Schedule/Action/Actdims.hpp>
 
+#include <opm/input/eclipse/EclipseState/EndpointScaling.hpp>
+#include <opm/input/eclipse/EclipseState/Phase.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/Regdims.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/Tabdims.hpp>
+
+#include <opm/input/eclipse/Schedule/Action/Actdims.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQParams.hpp>
+
+#include <bitset>
+#include <cstddef>
+#include <ctime>
 #include <optional>
-#include <string>
 
 namespace Opm {
-class Deck;
 
-class Phases {
+    class Deck;
+
+} // namespace Opm
+
+namespace Opm {
+
+class Phases
+{
 public:
     Phases() noexcept = default;
-    Phases( bool oil, bool gas, bool wat, bool solvent = false, bool polymer = false, bool energy = false,
-            bool polymw = false, bool foam = false, bool brine = false, bool zfraction = false ) noexcept;
+    Phases(bool oil, bool gas, bool wat,
+           bool solvent = false,
+           bool polymer = false,
+           bool energy = false,
+           bool polymw = false,
+           bool foam = false,
+           bool brine = false,
+           bool zfraction = false) noexcept;
 
     static Phases serializationTestObject();
 
@@ -53,10 +69,9 @@ public:
     }
 
 private:
-    std::bitset< NUM_PHASES_IN_ENUM > bits;
+    std::bitset<NUM_PHASES_IN_ENUM> bits;
 };
 
-
 class Welldims {
 public:
     Welldims() = default;
@@ -202,7 +217,21 @@ public:
         return this->nMaxNoBranchesConToNode;
     }
 
-    bool active() const;
+    bool extendedNetwork() const
+    {
+        return this->type_ == Type::Extended;
+    }
+
+    bool standardNetwork() const
+    {
+        return this->type_ == Type::Standard;
+    }
+
+    bool active() const
+    {
+        return this->extendedNetwork()
+            || this->standardNetwork();
+    }
 
     bool operator==(const NetworkDims& data) const;
 
@@ -215,9 +244,12 @@ public:
     }
 
 private:
+    enum class Type { None, Extended, Standard, };
+
     int nMaxNoNodes;
     int nMaxNoBranches;
     int nMaxNoBranchesConToNode;
+    Type type_{ Type::None };
 };
 
 class AquiferDimensions {
@@ -469,7 +501,11 @@ public:
     const SatFuncControls& saturationFunctionControls() const noexcept;
     const Nupcol& nupcol() const noexcept;
     const Tracers& tracers() const;
+    bool compositionalMode() const;
+    size_t numComps() const;
     bool co2Storage() const noexcept;
+    bool co2Sol() const noexcept;
+    bool h2Sol() const noexcept;
     bool h2Storage() const noexcept;
     bool micp() const noexcept;
     bool mech() const noexcept;
@@ -494,7 +530,10 @@ public:
         serializer(m_actdims);
         serializer(m_sfuncctrl);
         serializer(m_nupcol);
+        serializer(m_comps);
         serializer(m_co2storage);
+        serializer(m_co2sol);
+        serializer(m_h2sol);
         serializer(m_h2storage);
         serializer(m_micp);
         serializer(m_mech);
@@ -516,13 +555,17 @@ private:
     SatFuncControls m_sfuncctrl;
     Nupcol m_nupcol;
     Tracers m_tracers;
+    size_t m_comps = 0;
     bool m_co2storage;
+    bool m_co2sol;
+    bool m_h2sol;
     bool m_h2storage;
     bool m_micp;
     bool m_mech;
 };
 
+std::size_t declaredMaxRegionID(const Runspec& rspec);
 
-}
+} // namespace Opm
 
 #endif // OPM_RUNSPEC_HPP
