@@ -102,6 +102,7 @@ namespace Opm {
         const TableContainer& getSaltvdTables() const;
         const TableContainer& getSaltpvdTables() const;
         const TableContainer& getSaltsolTables() const;
+        const TableContainer& getPcfactTables() const;
         const TableContainer& getPermfactTables() const;
         const TableContainer& getEnkrvdTables() const;
         const TableContainer& getEnptvdTables() const;
@@ -201,6 +202,10 @@ namespace Opm {
 
         double salinity() const;
 
+        int actco2s() const;
+
+        bool diffMoleFraction() const;
+
         bool operator==(const TableManager& data) const;
 
         template<class Serializer>
@@ -261,6 +266,8 @@ namespace Opm {
             serializer(m_gas_comp_index);
             serializer(m_rtemp);
             serializer(m_salinity);
+            serializer(m_actco2s);
+            serializer(m_diff_mole_fraction);
             serializer(m_tlmixpar);
             serializer(m_ppcwmax);
             if (!serializer.isSerializing()) {
@@ -412,6 +419,8 @@ namespace Opm {
         std::size_t m_gas_comp_index = 77;
         double m_rtemp {288.7056}; // 60 Fahrenheit in Kelvin
         double m_salinity {0.0};
+        int m_actco2s {3};
+        bool m_diff_mole_fraction {true};
 
         struct SplitSimpleTables {
           size_t plyshMax = 0;
