@@ -21,25 +21,26 @@
 
 #include <cstddef>
 #include <ctime>
+#include <functional>
+#include <iosfwd>
 #include <map>
 #include <memory>
 #include <optional>
-#include <iosfwd>
 #include <set>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <opm/input/eclipse/EclipseState/Runspec.hpp>
+#include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
+#include <opm/input/eclipse/Schedule/Action/WGNames.hpp>
+#include <opm/input/eclipse/Schedule/CompletedCells.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
-#include <opm/input/eclipse/Schedule/MessageLimits.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleDeck.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleState.hpp>
+#include <opm/input/eclipse/Schedule/ScheduleStatic.hpp>
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
-#include <opm/input/eclipse/Schedule/CompletedCells.hpp>
-#include <opm/input/eclipse/Schedule/Action/WGNames.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 namespace Opm
@@ -53,6 +54,8 @@ namespace Opm
     class Deck;
     class DeckKeyword;
     class DeckRecord;
+    enum class ConnectionOrder;
+    class EclipseGrid;
     class EclipseState;
     class ErrorGuard;
     class FieldPropsManager;
@@ -60,12 +63,15 @@ namespace Opm
     class GTNode;
     class GuideRateConfig;
     class GuideRateModel;
+    class HandlerContext;
     enum class InputErrorAction;
     class ParseContext;
     class Python;
+    class Runspec;
     class RPTConfig;
+    class ScheduleGrid;
     class SCHEDULESection;
-    struct SimulatorUpdate;
+    class SegmentMatcher;
     class SummaryState;
     class TracerConfig;
     class UDQConfig;
@@ -74,84 +80,15 @@ namespace Opm
     class WellMatcher;
     enum class WellProducerCMode;
     enum class WellStatus;
+    class WelSegsSet;
     class WellTestConfig;
 
     namespace RestartIO { struct RstState; }
 
-
-    struct ScheduleStatic {
-        std::shared_ptr<const Python> m_python_handle;
-        std::string m_input_path;
-        ScheduleRestartInfo rst_info;
-        MessageLimits m_deck_message_limits;
-        UnitSystem m_unit_system;
-        Runspec m_runspec;
-        RSTConfig rst_config;
-        std::optional<int> output_interval;
-        double sumthin{-1.0};
-        bool rptonly{false};
-        bool gaslift_opt_active{false};
-
-        ScheduleStatic() = default;
-
-        explicit ScheduleStatic(std::shared_ptr<const Python> python_handle) :
-            m_python_handle(python_handle)
-        {}
-
-        ScheduleStatic(std::shared_ptr<const Python> python_handle,
-                       const ScheduleRestartInfo& restart_info,
-                       const Deck& deck,
-                       const Runspec& runspec,
-                       const std::optional<int>& output_interval_,
-                       const ParseContext& parseContext,
-                       ErrorGuard& errors);
-
-        template<class Serializer>
-        void serializeOp(Serializer& serializer)
-        {
-            serializer(m_deck_message_limits);
-            serializer(this->rst_info);
-            serializer(m_runspec);
-            serializer(m_unit_system);
-            serializer(this->m_input_path);
-            serializer(rst_info);
-            serializer(rst_config);
-            serializer(this->output_interval);
-            serializer(this->gaslift_opt_active);
-        }
-
-
-        static ScheduleStatic serializationTestObject();
-
-        bool operator==(const ScheduleStatic& other) const;
-    };
-
-
     class Schedule {
     public:
-
-        struct PairComp
-        {
-            bool operator()(const std::pair<std::string,KeywordLocation>& pair,
-                            const std::string& str) const
-            {
-                return std::get<0>(pair) < str;
-            }
-            bool operator()(const std::pair<std::string,KeywordLocation>& pair1,
-                            const std::pair<std::string,KeywordLocation>& pair2) const
-            {
-                return std::get<0>(pair1) < std::get<0>(pair2);
-            }
-            bool operator()(const std::string& str,
-                            const std::pair<std::string,KeywordLocation>& pair) const
-            {
-                return str < std::get<0>(pair);
-            }
-        };
-
-        using WelSegsSet = std::set<std::pair<std::string,KeywordLocation>,PairComp>;
-
         Schedule() = default;
+        ~Schedule() = default;
         explicit Schedule(std::shared_ptr<const Python> python_handle);
         Schedule(const Deck& deck,
                  const EclipseGrid& grid,
@@ -236,6 +173,7 @@ namespace Opm
         bool hasWell(const std::string& wellName, std::size_t timeStep) const;
 
         WellMatcher wellMatcher(std::size_t report_step) const;
+        std::function<std::unique_ptr<SegmentMatcher>()> segmentMatcherFactory(std::size_t report_step) const;
         std::vector<std::string> wellNames(const std::string& pattern, std::size_t timeStep, const std::vector<std::string>& matching_wells = {}) const;
         std::vector<std::string> wellNames(const std::string& pattern) const;
         std::vector<std::string> wellNames(std::size_t timeStep) const;
@@ -266,8 +204,12 @@ namespace Opm
         std::vector<Well> getWells(std::size_t timeStep) const;
         std::vector<Well> getWellsatEnd() const;
         void shut_well(const std::string& well_name, std::size_t report_step);
+        void shut_well(const std::string& well_name);
         void stop_well(const std::string& well_name, std::size_t report_step);
+        void stop_well(const std::string& well_name);
         void open_well(const std::string& well_name, std::size_t report_step);
+        void open_well(const std::string& well_name);
+        void clear_event(ScheduleEvents::Events, std::size_t report_step);
         void applyWellProdIndexScaling(const std::string& well_name, const std::size_t reportStep, const double scalingFactor);
 
         std::vector<const Group*> getChildGroups2(const std::string& group_name, std::size_t timeStep) const;
@@ -324,6 +266,7 @@ namespace Opm
         void create_next(const ScheduleBlock& block);
         void create_first(const time_point& start_time, const std::optional<time_point>& end_time);
 
+        void treat_critical_as_non_critical(bool value) { this->m_treat_critical_as_non_critical = value; }
 
         /*
           The cmp() function compares two schedule instances in a context aware
@@ -332,7 +275,8 @@ namespace Opm
           for the schedule instances created by loading a restart file.
         */
         static bool cmp(const Schedule& sched1, const Schedule& sched2, std::size_t report_step);
-        void applyKeywords(std::vector<DeckKeyword*>& keywords, std::size_t timeStep);
+        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords, std::size_t report_step);
+        void applyKeywords(std::vector<std::unique_ptr<DeckKeyword>>& keywords);
 
         template<class Serializer>
         void serializeOp(Serializer& serializer)
@@ -344,11 +288,15 @@ namespace Opm
             serializer(this->snapshots);
             serializer(this->restart_output);
             serializer(this->completed_cells);
+            serializer(this->m_treat_critical_as_non_critical);
+            serializer(this->current_report_step);
+            serializer(this->simUpdateFromPython);
 
             this->template pack_unpack<PAvg>(serializer);
             this->template pack_unpack<WellTestConfig>(serializer);
             this->template pack_unpack<GConSale>(serializer);
             this->template pack_unpack<GConSump>(serializer);
+            this->template pack_unpack<GroupEconProductionLimits>(serializer);
             this->template pack_unpack<WListManager>(serializer);
             this->template pack_unpack<Network::ExtNetwork>(serializer);
             this->template pack_unpack<Network::Balance>(serializer);
@@ -362,6 +310,8 @@ namespace Opm
             this->template pack_unpack<GasLiftOpt>(serializer);
             this->template pack_unpack<RFTConfig>(serializer);
             this->template pack_unpack<RSTConfig>(serializer);
+            this->template pack_unpack<ScheduleState::BHPDefaults>(serializer);
+            this->template pack_unpack<Source>(serializer);
 
             this->template pack_unpack_map<int, VFPProdTable>(serializer);
             this->template pack_unpack_map<int, VFPInjTable>(serializer);
@@ -504,82 +454,31 @@ namespace Opm
         void dump_deck(std::ostream& os) const;
 
     private:
-        struct HandlerContext {
-
-            const ScheduleBlock& block;
-            const DeckKeyword& keyword;
-            const std::size_t currentStep;
-            const std::vector<std::string>& matching_wells;
-            const bool actionx_mode;
-            const ParseContext& parseContext;
-            ErrorGuard& errors;
-            SimulatorUpdate * sim_update;
-            const std::unordered_map<std::string, double> * target_wellpi;
-            std::unordered_map<std::string, double>* wpimult_global_factor;
-            WelSegsSet *welsegs_wells;
-            std::set<std::string>*compsegs_wells;
-            const ScheduleGrid& grid;
-
-            /// \param welsegs_wells All wells with a WELSEGS entry for checks.
-            /// \param compegs_wells All wells with a COMPSEGS entry for checks.
-            HandlerContext(const ScheduleBlock& block_,
-                           const DeckKeyword& keyword_,
-                           const ScheduleGrid& grid_,
-                           const std::size_t currentStep_,
-                           const std::vector<std::string>& matching_wells_,
-                           bool actionx_mode_,
-                           const ParseContext& parseContext_,
-                           ErrorGuard& errors_,
-                           SimulatorUpdate * sim_update_,
-                           const std::unordered_map<std::string, double> * target_wellpi_,
-                           std::unordered_map<std::string, double>* wpimult_global_factor_,
-                           WelSegsSet* welsegs_wells_,
-                           std::set<std::string>* compsegs_wells_)
-            : block(block_)
-            , keyword(keyword_)
-            , currentStep(currentStep_)
-            , matching_wells(matching_wells_)
-            , actionx_mode(actionx_mode_)
-            , parseContext(parseContext_)
-            , errors(errors_)
-            , sim_update(sim_update_)
-            , target_wellpi(target_wellpi_)
-            , wpimult_global_factor(wpimult_global_factor_)
-            , welsegs_wells(welsegs_wells_)
-            , compsegs_wells(compsegs_wells_)
-            , grid(grid_)
-            {}
-
-            void affected_well(const std::string& well_name);
-
-            /// \brief Mark that the well occured in a  WELSEGS keyword
-            void welsegs_handled(const std::string& well_name)
-            {
-                if (welsegs_wells)
-                    welsegs_wells->insert({well_name, keyword.location()});
-            }
-
-            /// \brief Mark that the well occured in a  COMPSEGS keyword
-            void compsegs_handled(const std::string& well_name)
-            {
-                if (compsegs_wells)
-                    compsegs_wells->insert(well_name);
-            }
-
-        };
+        friend class HandlerContext;
 
         // Please update the member functions
         //   - operator==(const Schedule&) const
         //   - serializationTestObject()
         //   - serializeOp(Serializer&)
         // when you update/change this list of data members.
-        ScheduleStatic m_static;
-        ScheduleDeck m_sched_deck;
-        Action::WGNames action_wgnames;
-        std::optional<int> exit_status;
-        std::vector<ScheduleState> snapshots;
-        WriteRestartFileEvents restart_output;
-        CompletedCells completed_cells;
+        bool m_treat_critical_as_non_critical = false;
+        ScheduleStatic m_static{};
+        ScheduleDeck m_sched_deck{};
+        Action::WGNames action_wgnames{};
+        std::optional<int> exit_status{};
+        std::vector<ScheduleState> snapshots{};
+        WriteRestartFileEvents restart_output{};
+        CompletedCells completed_cells{};
+
+        // The current_report_step is set to the current report step when a PYACTION call is executed.
+        // This is needed since the Schedule object does not know the current report step of the simulator and
+        // we only allow PYACTIONS for the current and future report steps. 
+        std::size_t current_report_step = 0;
+        // The simUpdateFromPython points to a SimulatorUpdate collecting all updates from one PYACTION call.
+        // The SimulatorUpdate is reset before a new PYACTION call is executed.
+        // It is a shared_ptr, so a Schedule can be constructed using the copy constructor sharing the simUpdateFromPython.
+        // The copy constructor is needed for creating a mocked simulator (msim).
+        std::shared_ptr<SimulatorUpdate> simUpdateFromPython{};
 
         void load_rst(const RestartIO::RstState& rst,
                       const TracerConfig& tracer_config,
@@ -598,7 +497,7 @@ namespace Opm
                      int pvt_table,
                      WellGasInflowEquation gas_inflow,
                      std::size_t timeStep,
-                     Connection::Order wellConnectionOrder);
+                     ConnectionOrder wellConnectionOrder);
         bool updateWPAVE(const std::string& wname, std::size_t report_step, const PAvg& pavg);
 
         void updateGuideRateModel(const GuideRateModel& new_model, std::size_t report_step);
@@ -619,7 +518,8 @@ namespace Opm
         void addGroup(const std::string& groupName , std::size_t timeStep);
         void addGroup(Group group);
         void addGroup(const RestartIO::RstGroup& rst_group, std::size_t timeStep);
-        void addWell(const std::string& wellName, const DeckRecord& record, std::size_t timeStep, Connection::Order connection_order);
+        void addWell(const std::string& wellName, const DeckRecord& record,
+                    std::size_t timeStep, ConnectionOrder connection_order);
         void checkIfAllConnectionsIsShut(std::size_t currentStep);
         void end_report(std::size_t report_step);
         /// \param welsegs_wells All wells with a WELSEGS entry for checks.
@@ -634,131 +534,26 @@ namespace Opm
                            bool actionx_mode,
                            SimulatorUpdate* sim_update,
                            const std::unordered_map<std::string, double>* target_wellpi,
-                           std::unordered_map<std::string, double>* wpimult_global_factor = nullptr,
+                           std::unordered_map<std::string, double>& wpimult_global_factor,
                            WelSegsSet* welsegs_wells = nullptr,
                            std::set<std::string>* compsegs_wells = nullptr);
 
+        void internalWELLSTATUSACTIONXFromPYACTION(const std::string& well_name, std::size_t report_step, const std::string& wellStatus);
         void prefetch_cell_properties(const ScheduleGrid& grid, const DeckKeyword& keyword);
         void store_wgnames(const DeckKeyword& keyword);
-        std::vector<std::string> wellNames(const std::string& pattern, const HandlerContext& context);
+        std::vector<std::string> wellNames(const std::string& pattern,
+                                           const HandlerContext& context,
+                                           bool allowEmpty = false);
         std::vector<std::string> wellNames(const std::string& pattern, std::size_t timeStep, const std::vector<std::string>& matching_wells, InputErrorAction error_action, ErrorGuard& errors, const KeywordLocation& location) const;
-        void invalidNamePattern( const std::string& namePattern, const HandlerContext& context) const;
         static std::string formatDate(std::time_t t);
         std::string simulationDays(std::size_t currentStep) const;
         void applyGlobalWPIMULT( const std::unordered_map<std::string, double>& wpimult_global_factor);
 
         bool must_write_rst_file(std::size_t report_step) const;
 
-        void applyEXIT(const DeckKeyword&, std::size_t currentStep);
+        bool isWList(std::size_t report_step, const std::string& pattern) const;
+
         SimulatorUpdate applyAction(std::size_t reportStep, const std::string& action_name, const std::vector<std::string>& matching_wells);
-
-        /**
-         * Handles a "normal" keyword. A normal keyword is one that can be handled by a function with the standard set of arguments (the ones that are passed to this function).
-         *
-         * Normal keywords are found in the file KeywordHandlers.cpp; to add a new keyword handler to the file, add its signature in the list below,
-         * add the implementation to KeywordHandlers.cpp, and add a pointer to the handler in the dispatch registry in the implementation of this method, found at the bottom of
-         * KeywordHandlers.cpp.
-         *
-         * For the benefit of automatic cross-checking of the lists, all of these are in alphabetical order.
-         *
-         * @param handlerContext context object containing the environment in which the handler was invoked
-         * @param parseContext context object containing the parsing environment
-         * @param errors the error handling object for the current parsing process
-         *
-         * @return `true` if the keyword was handled
-         */
-        bool handleNormalKeyword(HandlerContext& handlerContext);
-
-        // Keyword Handlers
-        void handlePYACTION(const DeckKeyword&);
-        void handleWELPIRuntime(HandlerContext&);
-
-        // Normal keyword handlers -- in KeywordHandlers.cpp
-
-        void handleAQUCT     (HandlerContext&);
-        void handleAQUFETP   (HandlerContext&);
-        void handleAQUFLUX   (HandlerContext&);
-        void handleBRANPROP  (HandlerContext&);
-        void handleCOMPDAT   (HandlerContext&);
-        void handleCOMPLUMP  (HandlerContext&);
-        void handleCOMPORD   (HandlerContext&);
-        void handleCOMPSEGS  (HandlerContext&);
-        void handleCOMPTRAJ  (HandlerContext&);
-        void handleDRSDT     (HandlerContext&);
-        void handleDRSDTCON  (HandlerContext&);
-        void handleDRSDTR    (HandlerContext&);
-        void handleDRVDT     (HandlerContext&);
-        void handleDRVDTR    (HandlerContext&);
-        void handleEXIT      (HandlerContext&);
-        void handleGCONINJE  (HandlerContext&);
-        void handleGCONPROD  (HandlerContext&);
-        void handleGCONSALE  (HandlerContext&);
-        void handleGCONSUMP  (HandlerContext&);
-        void handleGEFAC     (HandlerContext&);
-        void handleGEOKeyword(HandlerContext&);
-        void handleGLIFTOPT  (HandlerContext&);
-        void handleGPMAINT   (HandlerContext&);
-        void handleGRUPNET   (HandlerContext&);
-        void handleGRUPTREE  (HandlerContext&);
-        void handleGUIDERAT  (HandlerContext&);
-        void handleLIFTOPT   (HandlerContext&);
-        void handleLINCOM    (HandlerContext&);
-        void handleMESSAGES  (HandlerContext&);
-        void handleMXUNSUPP  (HandlerContext&);
-        void handleNETBALAN  (HandlerContext&);
-        void handleNEXTSTEP  (HandlerContext&);
-        void handleNODEPROP  (HandlerContext&);
-        void handleNUPCOL    (HandlerContext&);
-        void handleRPTONLY   (HandlerContext&);
-        void handleRPTONLYO  (HandlerContext&);
-        void handleRPTRST    (HandlerContext&);
-        void handleRPTSCHED  (HandlerContext&);
-        void handleTUNING    (HandlerContext&);
-        void handleSAVE      (HandlerContext&);
-        void handleSUMTHIN   (HandlerContext&);
-        void handleUDQ       (HandlerContext&);
-        void handleVAPPARS   (HandlerContext&);
-        void handleVFPINJ    (HandlerContext&);
-        void handleVFPPROD   (HandlerContext&);
-        void handleWCONHIST  (HandlerContext&);
-        void handleWCONINJE  (HandlerContext&);
-        void handleWCONINJH  (HandlerContext&);
-        void handleWCONPROD  (HandlerContext&);
-        void handleWECON     (HandlerContext&);
-        void handleWEFAC     (HandlerContext&);
-        void handleWELOPEN   (HandlerContext&);
-        void handleWELPI     (HandlerContext&);
-        void handleWELSEGS   (HandlerContext&);
-        void handleWELSPECS  (HandlerContext&);
-        void handleWELTARG   (HandlerContext&);
-        void handleWELTRAJ   (HandlerContext&);
-        void handleWFOAM     (HandlerContext&);
-        void handleWGRUPCON  (HandlerContext&);
-        void handleWHISTCTL  (HandlerContext&);
-        void handleWINJTEMP  (HandlerContext&);
-        void handleWLIFTOPT  (HandlerContext&);
-        void handleWLIST     (HandlerContext&);
-        void handleWMICP     (HandlerContext&);
-        void handleWPAVE     (HandlerContext&);
-        void handleWPAVEDEP  (HandlerContext&);
-        void handleWVFPEXP   (HandlerContext&);
-        void handleWWPAVE    (HandlerContext&);
-        void handleWPIMULT   (HandlerContext&);
-        void handleWPMITAB   (HandlerContext&);
-        void handleWPOLYMER  (HandlerContext&);
-        void handleWRFT      (HandlerContext&);
-        void handleWRFTPLT   (HandlerContext&);
-        void handleWSALT     (HandlerContext&);
-        void handleWSEGITER  (HandlerContext&);
-        void handleWSEGSICD  (HandlerContext&);
-        void handleWSEGAICD  (HandlerContext&);
-        void handleWSEGVALV  (HandlerContext&);
-        void handleWSKPTAB   (HandlerContext&);
-        void handleWSOLVENT  (HandlerContext&);
-        void handleWTEMP     (HandlerContext&);
-        void handleWTEST     (HandlerContext&);
-        void handleWTMULT    (HandlerContext&);
-        void handleWTRACER   (HandlerContext&);
     };
 }
 
