@@ -26,6 +26,7 @@
 #include <opm/input/eclipse/Schedule/UDQ/UDQFunctionTable.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQInput.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQParams.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDT.hpp>
 
 #include <opm/input/eclipse/EclipseState/Util/OrderedMap.hpp>
 #include <opm/input/eclipse/EclipseState/Util/IOrderSet.hpp>
@@ -43,6 +44,7 @@ namespace Opm {
 
     class DeckRecord;
     class KeywordLocation;
+    class RegionSetMatcher;
     class Schedule;
     class SegmentMatcher;
     class SummaryState;
@@ -60,6 +62,7 @@ namespace Opm {
     class UDQConfig
     {
     public:
+        using RegionSetMatcherFactory = std::function<std::unique_ptr<RegionSetMatcher>()>;
         using SegmentMatcherFactory = std::function<std::unique_ptr<SegmentMatcher>()>;
 
         UDQConfig() = default;
@@ -101,6 +104,8 @@ namespace Opm {
                         const std::vector<std::string>& expression,
                         std::size_t                     report_step);
 
+        void add_table(const std::string& name, UDT udt);
+
         bool clear_pending_assignments();
 
         void eval_assign(std::size_t           report_step,
@@ -110,12 +115,13 @@ namespace Opm {
                          SummaryState&         st,
                          UDQState&             udq_state) const;
 
-        void eval(std::size_t           report_step,
-                  const Schedule&       sched,
-                  const WellMatcher&    wm,
-                  SegmentMatcherFactory create_segment_matcher,
-                  SummaryState&         st,
-                  UDQState&             udq_state) const;
+        void eval(std::size_t             report_step,
+                  const Schedule&         sched,
+                  const WellMatcher&      wm,
+                  SegmentMatcherFactory   create_segment_matcher,
+                  RegionSetMatcherFactory create_region_matcher,
+                  SummaryState&           st,
+                  UDQState&               udq_state) const;
 
         const UDQDefine& define(const std::string& key) const;
         const UDQAssign& assign(const std::string& key) const;
@@ -136,6 +142,7 @@ namespace Opm {
         std::vector<UDQAssign> assignments(UDQVarType var_type) const;
         const UDQParams& params() const;
         const UDQFunctionTable& function_table() const;
+        const std::unordered_map<std::string, UDT>& tables() const;
 
         bool operator==(const UDQConfig& config) const;
         void required_summary(std::unordered_set<std::string>& summary_keys) const;
@@ -146,6 +153,7 @@ namespace Opm {
             serializer(udq_params);
             serializer(m_definitions);
             serializer(m_assignments);
+            serializer(m_tables);
             serializer(units);
             serializer(input_index);
             serializer(type_count);
@@ -171,6 +179,7 @@ namespace Opm {
         // currently DEFINE'ed or ASSIGN'ed.
         std::unordered_map<std::string, UDQDefine> m_definitions;
         std::unordered_map<std::string, UDQAssign> m_assignments;
+        std::unordered_map<std::string, UDT> m_tables;
         std::unordered_map<std::string, std::string> units;
 
         IOrderSet<std::string> define_order;
