@@ -50,6 +50,7 @@ class DeckKeyword;
 class DeckRecord;
 class ErrorGuard;
 class EclipseGrid;
+class KeywordLocation;
 class ParseContext;
 class ScheduleGrid;
 class SICD;
@@ -68,6 +69,7 @@ class WellSegments;
 class WellTracerProperties;
 class WVFPEXP;
 class WVFPDP;
+class WDFAC;
 
 namespace RestartIO {
 struct RstWell;
@@ -167,8 +169,31 @@ public:
         static WellInjectionProperties serializationTestObject();
 
         void handleWELTARG(WELTARGCMode cmode, const UDAValue& new_arg, double SIFactorP);
-        void handleWCONINJE(const DeckRecord& record, bool availableForGroupControl, const std::string& well_name);
-        void handleWCONINJH(const DeckRecord& record, const bool is_producer, const std::string& well_name, const KeywordLocation& loc);
+
+        //! \brief Handle a WCONINJE keyword.
+        //! \param record The deck record to use
+        //! \param bhp_def The default BHP target in input units
+        //! \param availableForGroupControl True if available for group control
+        //! \param well_name Name of well
+        //! \param location Location of keyword for logging purpose
+        void handleWCONINJE(const DeckRecord& record,
+                            const double bhp_def,
+                            bool availableForGroupControl,
+                            const std::string& well_name,
+                            const KeywordLocation& location);
+
+        //! \brief Handle a WCONINJH keyword.
+        //! \param record The deck record to use
+        //! \param bhp_def The default BHP limit in SI units
+        //! \param is_producer True if well is a producer
+        //! \param well_name Name of well
+        //! \param loc Location of keyword for logging purpose
+        void handleWCONINJH(const DeckRecord& record,
+                            const double bhp_def,
+                            const bool is_producer,
+                            const std::string& well_name,
+                            const KeywordLocation& loc);
+
         bool hasInjectionControl(InjectorCMode controlModeArg) const {
             if (injectionControls & static_cast<int>(controlModeArg))
                 return true;
@@ -239,6 +264,7 @@ public:
         // BHP and THP limit
         double  bhp_hist_limit = 0.0;
         double  thp_hist_limit = 0.0;
+        bool    bhp_hist_limit_defaulted = true; // Tracks whether value was defaulted or not
 
         // historical BHP and THP under historical mode
         double  BHPH        = 0.0;
@@ -272,8 +298,30 @@ public:
 
         // this is used to check whether the specified control mode is an effective history matching production mode
         static bool effectiveHistoryProductionControl(ProducerCMode cmode);
-        void handleWCONPROD( const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const UnitSystem& unit_system, const std::string& well, const DeckRecord& record);
-        void handleWCONHIST( const std::optional<VFPProdTable::ALQ_TYPE>& alq_type, const UnitSystem& unit_system, const DeckRecord& record);
+
+        //! \brief Handle WCONPROD keyword.
+        //! \param alq_type ALQ type
+        //! \param bhp_def Default BHP target in SI units
+        //! \param unit_system Unit system to use
+        //! \param well Well name
+        //! \param record Deck record to use
+        //! \param location Location of keyword for logging purpose
+        void handleWCONPROD(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                            const double bhp_def,
+                            const UnitSystem& unit_system,
+                            const std::string& well,
+                            const DeckRecord& record,
+                            const KeywordLocation& location);
+
+        //! \brief Handle WCONHIST keyword.
+        //! \param alq_type ALQ type
+        //! \param bhp_def Default BHP limit in SI units
+        //! \param unit_system Unit system to use
+        //! \param record Deck record to use
+        void handleWCONHIST(const std::optional<VFPProdTable::ALQ_TYPE>& alq_type,
+                            const double bhp_def,
+                            const UnitSystem& unit_system,
+                            const DeckRecord& record);
         void handleWELTARG( WELTARGCMode cmode, const UDAValue& new_arg, double SiFactorP);
         void resetDefaultBHPLimit();
         void clearControls();
@@ -351,6 +399,7 @@ public:
 
     Well(const RestartIO::RstWell& rst_well,
          int report_step,
+         int rst_whistctl_cmode,
          const TracerConfig& tracer_config,
          const UnitSystem& unit_system,
          double udq_undefined);
@@ -410,6 +459,8 @@ public:
     const WellTracerProperties& getTracerProperties() const;
     const WVFPDP& getWVFPDP() const;
     const WVFPEXP& getWVFPEXP() const;
+    const WDFAC& getWDFAC() const;
+
     /* The rate of a given phase under the following assumptions:
      * * Returns zero if production is requested for an injector (and vice
      *   versa)
@@ -445,11 +496,11 @@ public:
     bool updatePrediction(bool prediction_mode);
     bool updateAutoShutin(bool auto_shutin);
     bool updateCrossFlow(bool allow_cross_flow);
-    bool updatePVTTable(int pvt_table);
-    bool updateHead(int I, int J);
+    bool updatePVTTable(std::optional<int> pvt_table);
+    bool updateHead(std::optional<int> I, std::optional<int> J);
     void updateRefDepth();
-    bool updateRefDepth(const std::optional<double>& ref_dpeth);
-    bool updateDrainageRadius(double drainage_radius);
+    bool updateRefDepth(std::optional<double> ref_dpeth);
+    bool updateDrainageRadius(std::optional<double> drainage_radius);
     void updateSegments(std::shared_ptr<WellSegments> segments_arg);
     bool updateConnections(std::shared_ptr<WellConnections> connections, bool force);
     bool updateConnections(std::shared_ptr<WellConnections> connections, const ScheduleGrid& grid);
@@ -457,7 +508,9 @@ public:
     bool updateGroup(const std::string& group);
     bool updateWellGuideRate(bool available, double guide_rate, GuideRateTarget guide_phase, double scale_factor);
     bool updateWellGuideRate(double guide_rate);
+    bool updateAvailableForGroupControl(bool available);
     bool updateEfficiencyFactor(double efficiency_factor);
+
     bool updateSolventFraction(double solvent_fraction);
     bool updateTracer(std::shared_ptr<WellTracerProperties> tracer_properties);
     bool updateFoamProperties(std::shared_ptr<WellFoamProperties> foam_properties);
@@ -475,11 +528,13 @@ public:
     void updateWPaveRefDepth(double ref_depth);
     bool updateWVFPDP(std::shared_ptr<WVFPDP> wvfpdp);
     bool updateWVFPEXP(std::shared_ptr<WVFPEXP> wvfpexp);
+    bool updateWDFAC(std::shared_ptr<WDFAC> wdfac);
+
 
     bool handleWELSEGS(const DeckKeyword& keyword);
     bool handleCOMPSEGS(const DeckKeyword& keyword, const ScheduleGrid& grid, const ParseContext& parseContext, ErrorGuard& errors);
     bool handleWELOPENConnections(const DeckRecord& record, Connection::State status);
-    bool handleCSKINConnections(const DeckRecord& record);
+    bool handleCSKIN(const DeckRecord& record, const KeywordLocation& location);
     bool handleCOMPLUMP(const DeckRecord& record);
     bool handleWPIMULT(const DeckRecord& record);
     bool handleWINJCLN(const DeckRecord& record, const KeywordLocation& location);
@@ -497,7 +552,7 @@ public:
     int fip_region_number() const;
     GasInflowEquation gas_inflow_equation() const;
     bool segmented_density_calculation() const { return true; }
-    double alq_value() const;
+    double alq_value(const SummaryState& st) const;
     double temperature() const;
     void setWellTemperature(const double temp);
     bool hasInjected( ) const;
@@ -539,6 +594,7 @@ public:
         serializer(has_produced);
         serializer(has_injected);
         serializer(prediction_mode);
+        serializer(derive_refdepth_from_conns_);
         serializer(econ_limits);
         serializer(foam_properties);
         serializer(polymer_properties);
@@ -550,6 +606,7 @@ public:
         serializer(injection);
         serializer(segments);
         serializer(wvfpdp);
+        serializer(wdfac);
         serializer(wvfpexp);
         serializer(m_pavg);
         serializer(well_temperature);
@@ -586,6 +643,7 @@ private:
     bool has_produced = false;
     bool has_injected = false;
     bool prediction_mode = true;
+    bool derive_refdepth_from_conns_ { true };
 
     std::shared_ptr<WellEconProductionLimits> econ_limits;
     std::shared_ptr<WellFoamProperties> foam_properties;
@@ -599,6 +657,8 @@ private:
     std::shared_ptr<WellSegments> segments;
     std::shared_ptr<WVFPDP> wvfpdp;
     std::shared_ptr<WVFPEXP> wvfpexp;
+    std::shared_ptr<WDFAC> wdfac;
+
     Status status;
     PAvg m_pavg;
     double well_temperature;
