@@ -74,7 +74,7 @@ public:
     /*!
      * \brief A human readable name for the water.
      */
-    static const char* name()
+    static std::string_view name()
     { return "H2O"; }
 
     /*!
