@@ -38,6 +38,7 @@
 #include <opm/material/common/MathToolbox.hpp>
 
 #include <cmath>
+#include <string_view>
 
 namespace Opm {
 
@@ -71,7 +72,7 @@ public:
     /*!
      * \brief A human readable name for the water.
      */
-    static const char* name()
+    static std::string_view name()
     { return "H2O"; }
 
     /*!
