@@ -32,18 +32,17 @@
 #define OPM_ECL_MATERIAL_LAW_MANAGER_HPP
 
 #include <opm/input/eclipse/EclipseState/Grid/FaceDir.hpp>
+#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
 
 #include <opm/material/fluidmatrixinteractions/SatCurveMultiplexer.hpp>
 #include <opm/material/fluidmatrixinteractions/EclEpsTwoPhaseLaw.hpp>
-#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
 #include <opm/material/fluidmatrixinteractions/EclHysteresisTwoPhaseLaw.hpp>
 #include <opm/material/fluidmatrixinteractions/EclMultiplexerMaterial.hpp>
 #include <opm/material/fluidmatrixinteractions/MaterialTraits.hpp>
 #include <opm/material/fluidmatrixinteractions/DirectionalMaterialLawParams.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/TableColumn.hpp>
-#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 
 #include <cassert>
+#include <functional>
 #include <memory>
 #include <tuple>
 #include <vector>
@@ -57,10 +56,12 @@ template<class Scalar> class EclEpsScalingPoints;
 template<class Scalar> struct EclEpsScalingPointsInfo;
 class EclHysteresisConfig;
 enum class EclTwoPhaseSystemType;
+class FieldPropsManager;
 class Runspec;
 class SgfnTable;
 class SgofTable;
 class SlgofTable;
+class TableColumn;
 
 /*!
  * \ingroup fluidmatrixinteractions
@@ -138,28 +139,43 @@ private:
     class InitParams {
     public:
         InitParams(EclMaterialLawManager<TraitsT>& parent, const EclipseState& eclState, size_t numCompressedElems);
-        void run();
+        // \brief Function argument 'fieldPropIntOnLeadAssigner' needed to lookup
+        //        field properties of cells on the leaf grid view for CpGrid with local grid refinement.
+        //        Function argument 'lookupIdxOnLevelZeroAssigner' is added to lookup, for each
+        //        leaf gridview cell with index 'elemIdx', its 'lookupIdx' (index of the parent/equivalent cell on level zero).
+        void run(const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>& fieldPropIntOnLeafAssigner,
+                 const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
     private:
         class HystParams;
-        void copySatnumArrays_();
-        void copyIntArray_(std::vector<int>& dest, const std::string keyword);
+        // \brief Function argument 'fieldPropIntOnLeadAssigner' needed to lookup
+        //        field properties of cells on the leaf grid view for CpGrid with local grid refinement.
+        void copySatnumArrays_(const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>&
+                               fieldPropIntOnLeafAssigner);
+        // \brief Function argument 'fieldPropIntOnLeadAssigner' needed to lookup
+        //        field properties of cells on the leaf grid view for CpGrid with local grid refinement.
+        void copyIntArray_(std::vector<int>& dest, const std::string keyword,
+                           const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>&
+                           fieldPropIntOnLeafAssigner);
         unsigned imbRegion_(std::vector<int>& array, unsigned elemIdx);
         void initArrays_(
-            std::vector<std::vector<int>*>& satnumArray,
-            std::vector<std::vector<int>*>& imbnumArray,
-            std::vector<std::vector<MaterialLawParams>*>& mlpArray);
+                         std::vector<std::vector<int>*>& satnumArray,
+                         std::vector<std::vector<int>*>& imbnumArray,
+                         std::vector<std::vector<MaterialLawParams>*>& mlpArray);
         void initMaterialLawParamVectors_();
         void initOilWaterScaledEpsInfo_();
-        void initSatnumRegionArray_();
+        // \brief Function argument 'fieldProptOnLeadAssigner' needed to lookup
+        //        field properties of cells on the leaf grid view for CpGrid with local grid refinement.
+        void initSatnumRegionArray_(const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>&
+                                    fieldPropIntOnLeafAssigner);
         void initThreePhaseParams_(
-            HystParams &hystParams,
-            MaterialLawParams& materialParams,
-            unsigned satRegionIdx,
-            unsigned elemIdx);
+                                   HystParams &hystParams,
+                                   MaterialLawParams& materialParams,
+                                   unsigned satRegionIdx,
+                                   unsigned elemIdx);
         void readEffectiveParameters_();
         void readUnscaledEpsPointsVectors_();
         template <class Container>
-          void readUnscaledEpsPoints_(Container& dest, std::shared_ptr<EclEpsConfig> config, EclTwoPhaseSystemType system_type);
+        void readUnscaledEpsPoints_(Container& dest, std::shared_ptr<EclEpsConfig> config, EclTwoPhaseSystemType system_type);
         unsigned satRegion_(std::vector<int>& array, unsigned elemIdx);
         unsigned satOrImbRegion_(std::vector<int>& array, std::vector<int>& default_vec, unsigned elemIdx);
 
@@ -172,23 +188,37 @@ private:
             std::shared_ptr<OilWaterTwoPhaseHystParams> getOilWaterParams();
             std::shared_ptr<GasWaterTwoPhaseHystParams> getGasWaterParams();
             void setConfig(unsigned satRegionIdx);
-            void setDrainageParamsOilGas(unsigned elemIdx, unsigned satRegionIdx);
-            void setDrainageParamsOilWater(unsigned elemIdx, unsigned satRegionIdx);
-            void setDrainageParamsGasWater(unsigned elemIdx, unsigned satRegionIdx);
-            void setImbibitionParamsOilGas(unsigned elemIdx, unsigned satRegionIdx);
-            void setImbibitionParamsOilWater(unsigned elemIdx, unsigned satRegionIdx);
-            void setImbibitionParamsGasWater(unsigned elemIdx, unsigned satRegionIdx);
+            // Function argument 'lookupIdxOnLevelZeroAssigner' is added to lookup, for each
+            // leaf gridview cell with index 'elemIdx', its 'lookupIdx' (index of the parent/equivalent cell on level zero).
+            void setDrainageParamsOilGas(unsigned elemIdx, unsigned satRegionIdx,
+                                         const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            void setDrainageParamsOilWater(unsigned elemIdx, unsigned satRegionIdx,
+                                           const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            void setDrainageParamsGasWater(unsigned elemIdx, unsigned satRegionIdx,
+                                           const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            void setImbibitionParamsOilGas(unsigned elemIdx, unsigned satRegionIdx,
+                                           const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            void setImbibitionParamsOilWater(unsigned elemIdx, unsigned satRegionIdx,
+                                             const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            void setImbibitionParamsGasWater(unsigned elemIdx, unsigned satRegionIdx,
+                                             const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
         private:
             bool hasGasWater_();
             bool hasGasOil_();
             bool hasOilWater_();
 
-            std::tuple<EclEpsScalingPointsInfo<Scalar>, EclEpsScalingPoints<Scalar>> readScaledEpsPoints_(
-              const EclEpsGridProperties& epsGridProperties, unsigned elemIdx, EclTwoPhaseSystemType type);
+            // Function argument 'lookupIdxOnLevelZeroAssigner' is added to lookup, for each
+            // leaf gridview cell with index 'elemIdx', its 'lookupIdx' (index of the parent/equivalent cell on level zero).
             std::tuple<EclEpsScalingPointsInfo<Scalar>, EclEpsScalingPoints<Scalar>>
-              readScaledEpsPointsDrainage_(unsigned elemIdx, EclTwoPhaseSystemType type);
+            readScaledEpsPoints_(const EclEpsGridProperties& epsGridProperties, unsigned elemIdx, EclTwoPhaseSystemType type,
+                                 const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
             std::tuple<EclEpsScalingPointsInfo<Scalar>, EclEpsScalingPoints<Scalar>>
-              readScaledEpsPointsImbibition_(unsigned elemIdx, EclTwoPhaseSystemType type);
+            readScaledEpsPointsDrainage_(unsigned elemIdx, EclTwoPhaseSystemType type,
+                                         const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+            std::tuple<EclEpsScalingPointsInfo<Scalar>, EclEpsScalingPoints<Scalar>>
+            readScaledEpsPointsImbibition_(unsigned elemIdx, EclTwoPhaseSystemType type,
+                                           const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
+
             EclMaterialLawManager<TraitsT>::InitParams& init_params_;
             EclMaterialLawManager<TraitsT>& parent_;
             const EclipseState& eclState_;
@@ -207,21 +237,21 @@ private:
             void readGasOilParameters_(GasOilEffectiveParamVector& dest, unsigned satRegionIdx);
             template <class TableType>
             void readGasOilFamily2_(
-                GasOilEffectiveTwoPhaseParams& effParams,
-                const Scalar Swco,
-                const double tolcrit,
-                const TableType& sofTable,
-                const SgfnTable& sgfnTable,
-                const std::string& columnName);
+                                    GasOilEffectiveTwoPhaseParams& effParams,
+                                    const Scalar Swco,
+                                    const double tolcrit,
+                                    const TableType& sofTable,
+                                    const SgfnTable& sgfnTable,
+                                    const std::string& columnName);
             void readGasOilSgof_(GasOilEffectiveTwoPhaseParams& effParams,
-                                const Scalar Swco,
-                                const double tolcrit,
-                                const SgofTable& sgofTable);
+                                 const Scalar Swco,
+                                 const double tolcrit,
+                                 const SgofTable& sgofTable);
 
             void readGasOilSlgof_(GasOilEffectiveTwoPhaseParams& effParams,
-                                const Scalar Swco,
-                                const double tolcrit,
-                                const SlgofTable& slgofTable);
+                                  const Scalar Swco,
+                                  const double tolcrit,
+                                  const SlgofTable& slgofTable);
             void readGasWaterParameters_(GasWaterEffectiveParamVector& dest, unsigned satRegionIdx);
             void readOilWaterParameters_(OilWaterEffectiveParamVector& dest, unsigned satRegionIdx);
 
@@ -242,7 +272,14 @@ private:
 public:
     void initFromState(const EclipseState& eclState);
 
-    void initParamsForElements(const EclipseState& eclState, size_t numCompressedElems);
+    // \brief Function argument 'fieldPropIntOnLeadAssigner' needed to lookup
+    //        field properties of cells on the leaf grid view for CpGrid with local grid refinement.
+    //        Function argument 'lookupIdxOnLevelZeroAssigner' is added to lookup, for each
+    //        leaf gridview cell with index 'elemIdx', its 'lookupIdx' (index of the parent/equivalent cell on level zero).
+    void initParamsForElements(const EclipseState& eclState, size_t numCompressedElems,
+                               const std::function<std::vector<int>(const FieldPropsManager&, const std::string&, bool)>&
+                               fieldPropIntOnLeafAssigner,
+                               const std::function<unsigned(unsigned)>& lookupIdxOnLevelZeroAssigner);
 
     /*!
      * \brief Modify the initial condition according to the SWATINIT keyword.
@@ -254,8 +291,18 @@ public:
      */
     std::pair<Scalar, bool>
     applySwatinit(unsigned elemIdx,
-                         Scalar pcow,
-                         Scalar Sw);
+                  Scalar pcow,
+                  Scalar Sw);
+
+    /// Apply SWATINIT-like scaling of oil/water capillary pressure curve at
+    /// simulation restart.
+    ///
+    /// \param[in] elemIdx Active cell index
+    ///
+    /// \param[in] maxPcow Scaled maximum oil/water capillary pressure.
+    ///   Typically the PPCW restart file array's entry for the
+    ///   corresponding cell.
+    void applyRestartSwatInit(const unsigned elemIdx, const Scalar maxPcow);
 
     bool enableEndPointScaling() const
     { return enableEndPointScaling_; }
