@@ -45,6 +45,8 @@
 #include <opm/material/binarycoefficients/Brine_CO2.hpp>
 #include <opm/material/binarycoefficients/H2O_N2.hpp>
 
+#include <string_view>
+
 namespace Opm {
 
 // Silence compiler warnings about use of variables
@@ -100,9 +102,9 @@ public:
     /*!
      * \copydoc BaseFluidSystem::phaseName
      */
-    static const char* phaseName(unsigned phaseIdx)
+    static std::string_view phaseName(unsigned phaseIdx)
     {
-        static const char* name[] = {
+        static const std::string_view name[] = {
             "liquid",
             "gas"
         };
@@ -167,9 +169,9 @@ public:
     /*!
      * \copydoc BaseFluidSystem::componentName
      */
-    static const char* componentName(unsigned compIdx)
+    static std::string_view componentName(unsigned compIdx)
     {
-        static const char* name[] = {
+        static const std::string_view name[] = {
             Brine::name(),
             CO2::name(),
         };
@@ -331,6 +333,10 @@ public:
         assert(temperature > 0);
         assert(pressure > 0);
 
+        // Activity model for salt-out effect in brine-CO2 mutual solubility
+        // 2 = Duan & Sun as modified in Spycher & Pruess, Trans. Porous Media, (2009)
+        const int activityModel = 3;
+
         // calulate the equilibrium composition for the given
         // temperature and pressure. TODO: calculateMoleFractions()
         // could use some cleanup.
@@ -341,7 +347,8 @@ public:
                                                     LhsEval(Brine_IAPWS::salinity),
                                                     /*knownPhaseIdx=*/-1,
                                                     xlCO2,
-                                                    xgH2O);
+                                                    xgH2O,
+                                                    activityModel);
 
         // normalize the phase compositions
         xlCO2 = max(0.0, min(1.0, xlCO2));
