@@ -41,14 +41,16 @@
 #include <opm/material/binarycoefficients/H2O_CO2.hpp>
 #include <opm/material/binarycoefficients/Brine_CO2.hpp>
 
+#include <opm/input/eclipse/EclipseState/Co2StoreConfig.hpp>
+
+
 #include <vector>
 
 namespace Opm {
 
-#if HAVE_ECL_INPUT
 class EclipseState;
 class Schedule;
-#endif
+class Co2StoreConfig;
 
 /*!
  * \brief This class represents the Pressure-Volume-Temperature relations of the liquid phase
@@ -78,6 +80,9 @@ public:
     explicit BrineCo2Pvt() = default;
 
     BrineCo2Pvt(const std::vector<Scalar>& salinity,
+                int activityModel = 3,
+                int thermalMixingModelSalt = 1,
+                int thermalMixingModelLiquid = 2,
                 Scalar T_ref = 288.71, //(273.15 + 15.56)
                 Scalar P_ref = 101325)
         : salinity_(salinity)
@@ -87,6 +92,8 @@ public:
             OPM_THROW(std::runtime_error, 
                 "BrineCo2Pvt class can only be used with default reference state (T, P) = (288.71 K, 1.01325e5 Pa)!");
         }
+        setActivityModelSalt(activityModel);
+        setThermalMixingModel(thermalMixingModelSalt, thermalMixingModelLiquid);
         int num_regions =  salinity_.size();
         co2ReferenceDensity_.resize(num_regions);
         brineReferenceDensity_.resize(num_regions);
@@ -154,6 +161,40 @@ public:
     void setEnableSaltConcentration(bool yesno)
     { enableSaltConcentration_ = yesno; }
 
+    /*!
+    * \brief Set activity coefficient model for salt in solubility model
+    */
+    void setActivityModelSalt(int activityModel)
+    {
+        // Only 1, 2 and 3 are allowed
+        if (activityModel > 3 || activityModel < 1) {
+            OPM_THROW(std::runtime_error, "The salt activity model options are 1, 2 or 3");
+        }
+        activityModel_ = activityModel;
+    }
+
+    /*!
+    * \brief Set thermal mixing model for co2 in brine
+    */
+    void setThermalMixingModel(int thermalMixingModelSalt, int thermalMixingModelLiquid)
+    {
+        if (thermalMixingModelSalt == 0)
+            saltMixType_ = Co2StoreConfig::SaltMixingType::NONE;
+        else if (thermalMixingModelSalt == 1)
+            saltMixType_ = Co2StoreConfig::SaltMixingType::MICHAELIDES;
+        else 
+            OPM_THROW(std::runtime_error, "The thermal mixing model option for salt are 0 or 1");
+
+        if (thermalMixingModelLiquid == 0)
+            liquidMixType_ = Co2StoreConfig::LiquidMixingType::NONE;
+        else if (thermalMixingModelLiquid == 1)
+            liquidMixType_ = Co2StoreConfig::LiquidMixingType::IDEAL;
+        else if (thermalMixingModelLiquid == 2)
+            liquidMixType_ = Co2StoreConfig::LiquidMixingType::DUANSUN;
+        else 
+            OPM_THROW(std::runtime_error, "The thermal mixing model option for liquid are 0, 1 and 2");
+    }
+
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
@@ -177,7 +218,7 @@ public:
                                        pressure,
                                        salinity,
                                        xlCO2)
-        - pressure / density_(regionIdx, temperature, pressure, Rs, salinity ));
+        - pressure / density(regionIdx, temperature, pressure, Rs, salinity ));
     }
     /*!
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
@@ -194,7 +235,7 @@ public:
                                        pressure,
                                        Evaluation(salinity_[regionIdx]),
                                        xlCO2)
-        - pressure / density_(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx])));
+        - pressure / density(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx])));
     }
 
     /*!
@@ -263,8 +304,8 @@ public:
     {
         OPM_TIMEFUNCTION_LOCAL();
         const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltconcentration);
-        Evaluation rsSat = rsSat_(regionIdx, temperature, pressure, salinity);
-        return (1.0 - convertRsToXoG_(rsSat,regionIdx)) * density_(regionIdx, temperature, pressure, rsSat, salinity)/brineReferenceDensity_[regionIdx];
+        Evaluation rs_sat = rsSat(regionIdx, temperature, pressure, salinity);
+        return (1.0 - convertRsToXoG_(rs_sat,regionIdx)) * density(regionIdx, temperature, pressure, rs_sat, salinity)/brineReferenceDensity_[regionIdx];
     }
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -278,7 +319,7 @@ public:
     {
         OPM_TIMEFUNCTION_LOCAL();
         const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
-        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density_(regionIdx, temperature, pressure, Rs, salinity)/brineReferenceDensity_[regionIdx];
+        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density(regionIdx, temperature, pressure, Rs, salinity)/brineReferenceDensity_[regionIdx];
     }
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -289,7 +330,7 @@ public:
                                             const Evaluation& pressure,
                                             const Evaluation& Rs) const
     {
-        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density_(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
+        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
     }
 
     /*!
@@ -300,9 +341,9 @@ public:
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        Evaluation rsSat = rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
-        return (1.0 - convertRsToXoG_(rsSat,regionIdx)) * density_(regionIdx, temperature, pressure, rsSat, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
+ 	OPM_TIMEFUNCTION_LOCAL();
+	Evaluation rs_sat = rsSat(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
+        return (1.0 - convertRsToXoG_(rs_sat,regionIdx)) * density(regionIdx, temperature, pressure, rs_sat, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
     }
 
     /*!
@@ -345,7 +386,7 @@ public:
                                              const Evaluation& /*maxOilSaturation*/) const
     {
         //TODO support VAPPARS
-        return rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
+        return rsSat(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
     /*!
@@ -358,7 +399,7 @@ public:
                                              const Evaluation& saltConcentration) const
     {
         const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
-        return rsSat_(regionIdx, temperature, pressure, salinity);
+        return rsSat(regionIdx, temperature, pressure, salinity);
     }
 
     /*!
@@ -369,7 +410,7 @@ public:
                                              const Evaluation& temperature,
                                              const Evaluation& pressure) const
     {
-        return rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
+        return rsSat(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
     const Scalar oilReferenceDensity(unsigned regionIdx) const
@@ -402,23 +443,16 @@ public:
         return pow(Evaluation(10), log_D_Brine) * 1e-4; // convert from cm2/s to m2/s
     }
 
-private:
-    std::vector<Scalar> brineReferenceDensity_;
-    std::vector<Scalar> co2ReferenceDensity_;
-    std::vector<Scalar> salinity_;
-    bool enableDissolution_ = true;
-    bool enableSaltConcentration_ = false;
-
-    template <class LhsEval>
-    LhsEval density_(unsigned regionIdx,
-                     const LhsEval& temperature,
-                     const LhsEval& pressure,
-                     const LhsEval& Rs,
-                     const LhsEval& salinity) const
+    template <class Evaluation>
+    Evaluation density(unsigned regionIdx,
+                       const Evaluation& temperature,
+                       const Evaluation& pressure,
+                       const Evaluation& Rs,
+                       const Evaluation& salinity) const
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        LhsEval xlCO2 = convertXoGToxoG_(convertRsToXoG_(Rs,regionIdx), salinity);
-        LhsEval result = liquidDensity_(temperature,
+	OPM_TIMEFUNCTION_LOCAL();        
+	Evaluation xlCO2 = convertXoGToxoG_(convertRsToXoG_(Rs,regionIdx), salinity);
+        Evaluation result = liquidDensity_(temperature,
                                         pressure,
                                         xlCO2,
                                         salinity);
@@ -427,6 +461,44 @@ private:
         return result;
     }
 
+    template <class Evaluation>
+    Evaluation rsSat(unsigned regionIdx,
+                     const Evaluation& temperature,
+                     const Evaluation& pressure,
+                     const Evaluation& salinity) const
+    {
+	    OPM_TIMEFUNCTION_LOCAL();
+        if (!enableDissolution_)
+            return 0.0;
+
+        // calulate the equilibrium composition for the given
+        // temperature and pressure. 
+        Evaluation xgH2O;
+        Evaluation xlCO2;
+        BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
+                                                    pressure,
+                                                    salinity,
+                                                    /*knownPhaseIdx=*/-1,
+                                                    xlCO2,
+                                                    xgH2O,
+                                                    activityModel_,
+                                                    extrapolate);
+
+        // normalize the phase compositions
+        xlCO2 = max(0.0, min(1.0, xlCO2));
+
+        return convertXoGToRs(convertxoGToXoG(xlCO2, salinity), regionIdx);
+    }
+
+private:
+    std::vector<Scalar> brineReferenceDensity_;
+    std::vector<Scalar> co2ReferenceDensity_;
+    std::vector<Scalar> salinity_;
+    bool enableDissolution_ = true;
+    bool enableSaltConcentration_ = false;
+    int activityModel_;
+    Co2StoreConfig::LiquidMixingType liquidMixType_; 
+    Co2StoreConfig::SaltMixingType saltMixType_; 
 
     template <class LhsEval>
     LhsEval liquidDensity_(const LhsEval& T,
@@ -542,101 +614,87 @@ private:
         return XoG/(1.0 - XoG)*(rho_oRef/rho_gRef);
     }
 
-
     template <class LhsEval>
-    LhsEval rsSat_(unsigned regionIdx,
-                   const LhsEval& temperature,
-                   const LhsEval& pressure,
-                   const LhsEval& salinity) const
+    LhsEval liquidEnthalpyBrineCO2_(const LhsEval& T,
+                                    const LhsEval& p,
+                                    const LhsEval& salinity, 
+                                    const LhsEval& X_CO2_w) const
     {
-        OPM_TIMEFUNCTION_LOCAL();
-        if (!enableDissolution_)
-            return 0.0;
 
-        // calulate the equilibrium composition for the given
-        // temperature and pressure.
-        LhsEval xgH2O;
-        LhsEval xlCO2;
-        BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
-                                                    pressure,
-                                                    salinity,
-                                                    /*knownPhaseIdx=*/-1,
-                                                    xlCO2,
-                                                    xgH2O,
-                                                    extrapolate);
 
-        // normalize the phase compositions
-        xlCO2 = max(0.0, min(1.0, xlCO2));
+        if (liquidMixType_ == Co2StoreConfig::LiquidMixingType::NONE 
+            && saltMixType_ == Co2StoreConfig::SaltMixingType::NONE)
+            return H2O::liquidEnthalpy(T, p);
 
-        return convertXoGToRs(convertxoGToXoG(xlCO2, salinity), regionIdx);
-    }
+        LhsEval hw = H2O::liquidEnthalpy(T, p) /1E3; /* kJ/kg */
+        LhsEval h_ls1 = hw;
+        // Use mixing model for salt by MICHAELIDES
+        if (saltMixType_ == Co2StoreConfig::SaltMixingType::MICHAELIDES) {
 
-    template <class LhsEval>
-    static LhsEval liquidEnthalpyBrineCO2_(const LhsEval& T,
-                                           const LhsEval& p,
-                                           const LhsEval& salinity,
-                                           const LhsEval& X_CO2_w)
-    {
-        OPM_TIMEFUNCTION_LOCAL();
-        /* X_CO2_w : mass fraction of CO2 in brine */
+            /* X_CO2_w : mass fraction of CO2 in brine */
 
-        /* same function as enthalpy_brine, only extended by CO2 content */
+            /* same function as enthalpy_brine, only extended by CO2 content */
 
-        /*Numerical coefficents from PALLISER*/
-        static constexpr Scalar f[] = {
-            2.63500E-1, 7.48368E-6, 1.44611E-6, -3.80860E-10
-        };
+            /*Numerical coefficents from PALLISER*/
+            static constexpr Scalar f[] = {
+                2.63500E-1, 7.48368E-6, 1.44611E-6, -3.80860E-10
+            };
 
-        /*Numerical coefficents from MICHAELIDES for the enthalpy of brine*/
-        static constexpr Scalar a[4][3] = {
-            { 9633.6, -4080.0, +286.49 },
-            { +166.58, +68.577, -4.6856 },
-            { -0.90963, -0.36524, +0.249667E-1 },
-            { +0.17965E-2, +0.71924E-3, -0.4900E-4 }
-        };
+            /*Numerical coefficents from MICHAELIDES for the enthalpy of brine*/
+            static constexpr Scalar a[4][3] = {
+                { 9633.6, -4080.0, +286.49 },
+                { +166.58, +68.577, -4.6856 },
+                { -0.90963, -0.36524, +0.249667E-1 },
+                { +0.17965E-2, +0.71924E-3, -0.4900E-4 }
+            };
 
-        LhsEval theta, h_NaCl;
-        LhsEval h_ls1, d_h;
-        LhsEval delta_h;
-        LhsEval delta_hCO2, hg, hw;
+            LhsEval theta, h_NaCl;
+            LhsEval d_h, delta_h;
 
-        theta = T - 273.15;
+            theta = T - 273.15;
 
-        // Regularization
-        Scalar scalarTheta = scalarValue(theta);
-        Scalar S_lSAT = f[0] + scalarTheta*(f[1] + scalarTheta*(f[2] + scalarTheta*f[3]));
+            // Regularization
+            Scalar scalarTheta = scalarValue(theta);
+            Scalar S_lSAT = f[0] + scalarTheta*(f[1] + scalarTheta*(f[2] + scalarTheta*f[3]));
 
-        LhsEval S = salinity;
-        if (S > S_lSAT)
-            S = S_lSAT;
+            LhsEval S = salinity;
+            if (S > S_lSAT)
+                S = S_lSAT;
 
-        hw = H2O::liquidEnthalpy(T, p) /1E3; /* kJ/kg */
+            /*DAUBERT and DANNER*/
+            /*U=*/h_NaCl = (3.6710E4*T + 0.5*(6.2770E1)*T*T - ((6.6670E-2)/3)*T*T*T
+                            +((2.8000E-5)/4)*(T*T*T*T))/(58.44E3)- 2.045698e+02; /* kJ/kg */
 
-        /*DAUBERT and DANNER*/
-        /*U=*/h_NaCl = (3.6710E4*T + 0.5*(6.2770E1)*T*T - ((6.6670E-2)/3)*T*T*T
-                        +((2.8000E-5)/4)*(T*T*T*T))/(58.44E3)- 2.045698e+02; /* kJ/kg */
+            LhsEval m = 1E3/58.44 * S/(1-S);
+            int i = 0;
+            int j = 0;
+            d_h = 0;
 
-        LhsEval m = 1E3/58.44 * S/(1-S);
-        int i = 0;
-        int j = 0;
-        d_h = 0;
-
-        for (i = 0; i<=3; i++) {
-            for (j=0; j<=2; j++) {
-                d_h = d_h + a[i][j] * pow(theta, static_cast<Scalar>(i)) * pow(m, j);
+            for (i = 0; i<=3; i++) {
+                for (j=0; j<=2; j++) {
+                    d_h = d_h + a[i][j] * pow(theta, static_cast<Scalar>(i)) * pow(m, j);
+                }
             }
+            /* heat of dissolution for halite according to Michaelides 1971 */
+            delta_h = (4.184/(1E3 + (58.44 * m)))*d_h;
+
+            /* Enthalpy of brine without CO2 */
+            h_ls1 =(1-S)*hw + S*h_NaCl + S*delta_h; /* kJ/kg */
+
+            // Use Enthalpy of brine without CO2
+            if (liquidMixType_ == Co2StoreConfig::LiquidMixingType::NONE)
+                return  h_ls1*1E3;
         }
-        /* heat of dissolution for halite according to Michaelides 1971 */
-        delta_h = (4.184/(1E3 + (58.44 * m)))*d_h;
-
-        /* Enthalpy of brine without CO2 */
-        h_ls1 =(1-S)*hw + S*h_NaCl + S*delta_h; /* kJ/kg */
 
+        LhsEval delta_hCO2, hg;
         /* heat of dissolution for CO2 according to Fig. 6 in Duan and Sun 2003. (kJ/kg)
            In the relevant temperature ranges CO2 dissolution is
            exothermal */
-        delta_hCO2 = (-57.4375 + T * 0.1325) * 1000/44;
-
+        if (liquidMixType_ == Co2StoreConfig::LiquidMixingType::DUANSUN)
+            delta_hCO2 = (-57.4375 + T * 0.1325) * 1000/44;
+        else 
+            delta_hCO2 = 0.0;
+            
         /* enthalpy contribution of CO2 (kJ/kg) */
         hg = CO2::gasEnthalpy(T, p, extrapolate)/1E3 + delta_hCO2;
 
