@@ -35,15 +35,15 @@
 #include <opm/material/components/SimpleHuDuanH2O.hpp>
 #include <opm/material/common/UniformTabulated2DFunction.hpp>
 #include <opm/material/binarycoefficients/Brine_CO2.hpp>
+#include <opm/input/eclipse/EclipseState/Co2StoreConfig.hpp>
 
 #include <vector>
 
 namespace Opm {
 
-#if HAVE_ECL_INPUT
 class EclipseState;
 class Schedule;
-#endif
+class Co2StoreConfig;
 
 /*!
  * \brief This class represents the Pressure-Volume-Temperature relations of the gas phase
@@ -64,6 +64,8 @@ public:
     explicit Co2GasPvt() = default;
 
     Co2GasPvt(const std::vector<Scalar>& salinity,
+              int activityModel = 3,
+              int thermalMixingModel = 1,
               Scalar T_ref = 288.71, //(273.15 + 15.56)
               Scalar P_ref = 101325)
         : salinity_(salinity)
@@ -73,6 +75,9 @@ public:
             OPM_THROW(std::runtime_error, 
                 "BrineCo2Pvt class can only be used with default reference state (T, P) = (288.71 K, 1.01325e5 Pa)!");
         }
+        setActivityModelSalt(activityModel);
+        setThermalMixingModel(thermalMixingModel);
+        
         int num_regions = salinity_.size();
         setNumRegions(num_regions);
         for (int i = 0; i < num_regions; ++i) {
@@ -80,12 +85,10 @@ public:
             brineReferenceDensity_[i] = Brine::liquidDensity(T_ref, P_ref, salinity_[i], extrapolate);
         }
     }
-#if HAVE_ECL_INPUT
     /*!
      * \brief Initialize the parameters for CO2 gas using an ECL deck.
      */
     void initFromState(const EclipseState& eclState, const Schedule&);
-#endif
 
     void setNumRegions(size_t numRegions)
     {
@@ -119,6 +122,35 @@ public:
     void setEnableVaporizationWater(bool yesno)
     { enableVaporization_ = yesno; }
 
+    /*!
+    * \brief Set activity coefficient model for salt in solubility model
+    */
+    void setActivityModelSalt(int activityModel)
+    {
+        // Only 1, 2, and 3 are allowed
+        if (activityModel > 3 || activityModel < 1) {
+            OPM_THROW(std::runtime_error, "The salt activity model options are 1, 2 or 3");
+        }
+        activityModel_ = activityModel;
+    }
+
+   /*!
+    * \brief Set thermal mixing model for co2 in brine
+    */
+    void setThermalMixingModel(int thermalMixingModel)
+    {
+        // Only 0 and 1 are allowed
+        // 0 = Use pure CO2 entalpy
+        // 1 = Account for vapporized water in gas phase (Mass fraction)
+        if (thermalMixingModel == 0)
+            gastype_ = Co2StoreConfig::GasMixingType::NONE;
+
+        else if (thermalMixingModel == 1)
+            gastype_ = Co2StoreConfig::GasMixingType::IDEAL;
+        else
+            OPM_THROW(std::runtime_error, "The thermal mixing model options are 0 and 1");
+    }
+
     /*!
      * \brief Finish initializing the co2 phase PVT properties.
      */
@@ -144,9 +176,14 @@ public:
                         const Evaluation& rvw) const
     {
         OPM_TIMEBLOCK_LOCAL(internalEnergy);
-        // assume ideal mixture
-        Evaluation result = 0;
 
+        if (gastype_ == Co2StoreConfig::GasMixingType::NONE)
+            // use the gasInternalEnergy of CO2
+            return CO2::gasInternalEnergy(temperature, pressure, extrapolate);  
+
+        assert(gastype_ == Co2StoreConfig::GasMixingType::IDEAL);
+        // account for H2O in the gas phase
+        Evaluation result = 0;
         // The CO2STORE option both works for GAS/WATER and GAS/OIL systems
         // Either rv og rvw should be zero
         assert(rv == 0.0 || rvw == 0.0);
@@ -194,14 +231,15 @@ public:
         if (!enableVaporization_)
             return CO2::gasDensity(temperature, pressure, extrapolate)/gasReferenceDensity_[regionIdx];
 
-        // assume ideal mixture
-        // The CO2STORE option both works for GAS/WATER and GAS/OIL systems
-        // Either rv og rvw should be zero
-        assert(rv == 0.0 || rvw == 0.0);
-        const Evaluation xBrine = convertRvwToXgW_(max(rvw,rv),regionIdx);
+        // Use CO2 density for the gas phase.
         const auto& rhoCo2 = CO2::gasDensity(temperature, pressure, extrapolate);
-        const auto& rhoH2O = H2O::gasDensity(temperature, pressure);
-        return 1.0 / ( ( xBrine/rhoH2O + (1.0 - xBrine)/rhoCo2) * gasReferenceDensity_[regionIdx]);
+        //const auto& rhoH2O = H2O::gasDensity(temperature, pressure);
+        //The CO2STORE option both works for GAS/WATER and GAS/OIL systems
+        //Either rv og rvw should be zero
+        //assert(rv == 0.0 || rvw == 0.0);
+        //const Evaluation xBrine = convertRvwToXgW_(max(rvw,rv),regionIdx);
+        //const auto rho = 1.0/(xBrine/rhoH2O + (1.0 - xBrine)/rhoCo2);
+        return rhoCo2/(gasReferenceDensity_[regionIdx] + max(rvw,rv)*brineReferenceDensity_[regionIdx]);
     }
 
     /*!
@@ -314,6 +352,7 @@ private:
                                                     /*knownPhaseIdx=*/-1,
                                                     xlCO2,
                                                     xgH2O,
+                                                    activityModel_,
                                                     extrapolate);
 
         // normalize the phase compositions
@@ -371,6 +410,8 @@ private:
     std::vector<Scalar> gasReferenceDensity_;
     std::vector<Scalar> salinity_;
     bool enableVaporization_ = true;
+    int activityModel_;
+    Co2StoreConfig::GasMixingType gastype_; 
 };
 
 } // namespace Opm
