@@ -28,7 +28,6 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
-#include <initializer_list>
 #include <map>
 #include <stdexcept>
 #include <string>
@@ -36,9 +35,7 @@
 #include <unordered_map>
 #include <vector>
 
-namespace Opm {
-
-    namespace data {
+namespace Opm { namespace data {
 
     class Rates {
         /* Methods are defined inline for performance, as the actual *work* done
@@ -196,6 +193,53 @@ namespace Opm {
             double vaporized_water = 0.0;
     };
 
+    struct ConnectionFiltrate {
+        double rate;
+        double total;
+        double skin_factor;
+        double thickness;
+        double perm;
+        double poro;
+        double radius;
+        double area_of_flow;
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer) {
+            serializer(rate);
+            serializer(total);
+            serializer(skin_factor);
+            serializer(thickness);
+            serializer(perm);
+            serializer(poro);
+            serializer(radius);
+            serializer(area_of_flow);
+        }
+
+        bool operator==(const ConnectionFiltrate& filtrate) const
+        {
+            return this->rate == filtrate.rate &&
+                   this->total == filtrate.total &&
+                   this->skin_factor == filtrate.skin_factor &&
+                   this->thickness == filtrate.thickness &&
+                   this->perm == filtrate.perm &&
+                   this->poro == filtrate.poro &&
+                   this->radius == filtrate.radius &&
+                   this->area_of_flow == filtrate.area_of_flow;
+        }
+
+        static ConnectionFiltrate serializationTestObject()
+        {
+            return {0.8, 100., -1., 2., 1.e-9,
+                    0.3, 0.05, 0.8};
+        }
+
+        template <class MessageBufferType>
+        void write(MessageBufferType& buffer) const;
+
+        template <class MessageBufferType>
+        void read(MessageBufferType& buffer);
+    };
+
     struct Connection {
         using global_index = size_t;
         static const constexpr int restart_size = 6;
@@ -209,18 +253,26 @@ namespace Opm {
         double cell_saturation_gas;
         double effective_Kh;
         double trans_factor;
+        double d_factor;
+        double compact_mult{1.0}; // Rock compaction transmissibility multiplier (ROCKTAB)
+
+        ConnectionFiltrate filtrate;
 
         bool operator==(const Connection& conn2) const
         {
-            return index == conn2.index &&
-                   rates == conn2.rates &&
-                   pressure == conn2.pressure &&
-                   reservoir_rate == conn2.reservoir_rate &&
-                   cell_pressure == conn2.cell_pressure &&
-                   cell_saturation_water == conn2.cell_saturation_water &&
-                   cell_saturation_gas == conn2.cell_saturation_gas &&
-                   effective_Kh == conn2.effective_Kh &&
-                   trans_factor == conn2.trans_factor;
+            return (index == conn2.index)
+                && (rates == conn2.rates)
+                && (pressure == conn2.pressure)
+                && (reservoir_rate == conn2.reservoir_rate)
+                && (cell_pressure == conn2.cell_pressure)
+                && (cell_saturation_water == conn2.cell_saturation_water)
+                && (cell_saturation_gas == conn2.cell_saturation_gas)
+                && (effective_Kh == conn2.effective_Kh)
+                && (trans_factor == conn2.trans_factor)
+                && (d_factor == conn2.d_factor)
+                && (compact_mult == conn2.compact_mult)
+                && (filtrate == conn2.filtrate)
+                ;
         }
 
         template <class MessageBufferType>
@@ -242,13 +294,19 @@ namespace Opm {
             serializer(cell_saturation_gas);
             serializer(effective_Kh);
             serializer(trans_factor);
+            serializer(d_factor);
+            serializer(compact_mult);
+            serializer(filtrate);
         }
 
         static Connection serializationTestObject()
         {
-            return Connection{1, Rates::serializationTestObject(),
-                              2.0, 3.0, 4.0, 5.0,
-                              6.0, 7.0, 8.0};
+            return Connection {
+                1, Rates::serializationTestObject(),
+                2.0, 3.0, 4.0, 5.0,
+                6.0, 7.0, 8.0, 9.0, 0.987,
+                ConnectionFiltrate::serializationTestObject()
+            };
         }
     };
 
@@ -318,15 +376,11 @@ namespace Opm {
         }
     };
 
-    class SegmentPhaseQuantity
+    template <typename Items>
+    class SegmentQuantity
     {
     public:
-        enum class Item : std::size_t {
-            Oil, Gas, Water,
-
-            // -- Must be last enumerator --
-            NumItems,
-        };
+        using Item = typename Items::Item;
 
         void clear()
         {
@@ -341,7 +395,7 @@ namespace Opm {
             return (i < Size) && this->hasItem(i);
         }
 
-        bool operator==(const SegmentPhaseQuantity& vec) const
+        bool operator==(const SegmentQuantity& vec) const
         {
             return (this->has_   == vec.has_)
                 && (this->value_ == vec.value_);
@@ -351,21 +405,21 @@ namespace Opm {
         {
             if (! this->has(p)) {
                 throw std::invalid_argument {
-                    "Request for Unset Item Value for " + this->itemName(p)
+                    "Request for Unset Item Value for " + Items::itemName(p)
                 };
             }
 
             return this->value_[ this->index(p) ];
         }
 
-        SegmentPhaseQuantity& set(const Item p, const double value)
+        SegmentQuantity& set(const Item p, const double value)
         {
             const auto i = this->index(p);
 
             if (i >= Size) {
                 throw std::invalid_argument {
                     "Cannot Assign Item Value for Unsupported Item '"
-                    + this->itemName(p) + '\''
+                    + Items::itemName(p) + '\''
                 };
             }
 
@@ -403,12 +457,15 @@ namespace Opm {
             serializer(this->value_);
         }
 
-        static SegmentPhaseQuantity serializationTestObject()
+        static SegmentQuantity serializationTestObject()
         {
-            return SegmentPhaseQuantity{}
-                .set(Item::Oil  , 1.0)
-                .set(Item::Gas  , 7.0)
-                .set(Item::Water, 2.9);
+            auto quant = SegmentQuantity{};
+
+            for (const auto& [item, value] : Items::serializationTestItems()) {
+                quant.set(item, value);
+            }
+
+            return quant;
         }
 
     private:
@@ -430,8 +487,18 @@ namespace Opm {
         {
             return (this->has_ & (1 << i)) != 0;
         }
+    };
 
-        std::string itemName(const Item p) const
+    struct PhaseItems
+    {
+        enum class Item {
+            Oil, Gas, Water,
+
+            // -- Must be last enumerator --
+            NumItems,
+        };
+
+        static std::string itemName(const Item p)
         {
             switch (p) {
             case Item::Oil:   return "Oil";
@@ -442,10 +509,59 @@ namespace Opm {
                 return "Out of bounds (NumItems)";
             }
 
-            return "Unknown (" + std::to_string(this->index(p)) + ')';
+            return "Unknown (" + std::to_string(static_cast<int>(p)) + ')';
+        }
+
+        static auto serializationTestItems()
+        {
+            return std::vector {
+                std::pair { Item::Oil  , 1.0 },
+                std::pair { Item::Gas  , 7.0 },
+                std::pair { Item::Water, 2.9 },
+            };
         }
     };
 
+    struct DensityItems
+    {
+        enum class Item {
+            Oil, Gas, Water, Mixture, MixtureWithExponents,
+
+            // -- Must be last enumerator --
+            NumItems,
+        };
+
+        static std::string itemName(const Item p)
+        {
+            switch (p) {
+            case Item::Oil:                  return "Oil";
+            case Item::Gas:                  return "Gas";
+            case Item::Water:                return "Water";
+            case Item::Mixture:              return "Mixture";
+            case Item::MixtureWithExponents: return "MixtureWithExponents";
+
+            case Item::NumItems:
+                return "Out of bounds (NumItems)";
+            }
+
+            return "Unknown (" + std::to_string(static_cast<int>(p)) + ')';
+        }
+
+        static auto serializationTestItems()
+        {
+            return std::vector {
+                std::pair { Item::Oil                 , 876.54 },
+                std::pair { Item::Gas                 , 321.09 },
+                std::pair { Item::Water               , 987.65 },
+                std::pair { Item::Mixture             , 975.31 },
+                std::pair { Item::MixtureWithExponents, 765.43 },
+            };
+        }
+    };
+
+    using SegmentPhaseQuantity = SegmentQuantity<PhaseItems>;
+    using SegmentPhaseDensity = SegmentQuantity<DensityItems>;
+
     struct Segment
     {
         Rates rates{};
@@ -453,6 +569,7 @@ namespace Opm {
         SegmentPhaseQuantity velocity{};
         SegmentPhaseQuantity holdup{};
         SegmentPhaseQuantity viscosity{};
+        SegmentPhaseDensity density{};
         std::size_t segNumber{};
 
         bool operator==(const Segment& seg2) const
@@ -462,6 +579,7 @@ namespace Opm {
                 && (velocity == seg2.velocity)
                 && (holdup == seg2.holdup)
                 && (viscosity == seg2.viscosity)
+                && (density == seg2.density)
                 && (segNumber == seg2.segNumber);
         }
 
@@ -479,6 +597,7 @@ namespace Opm {
             serializer(this->velocity);
             serializer(this->holdup);
             serializer(this->viscosity);
+            serializer(this->density);
             serializer(this->segNumber);
         }
 
@@ -490,6 +609,7 @@ namespace Opm {
                 SegmentPhaseQuantity::serializationTestObject(), // velocity
                 SegmentPhaseQuantity::serializationTestObject(), // holdup
                 SegmentPhaseQuantity::serializationTestObject(), // viscosity
+                SegmentPhaseDensity::serializationTestObject(),  // density
                 10
             };
         }
@@ -549,6 +669,90 @@ namespace Opm {
         }
     };
 
+    class WellBlockAvgPress
+    {
+    public:
+        enum class Quantity { WBP, WBP4, WBP5, WBP9 };
+
+        double& operator[](const Quantity q)
+        {
+            return this->wbp_[static_cast<std::size_t>(q)];
+        }
+
+        double operator[](const Quantity q) const
+        {
+            return this->wbp_[static_cast<std::size_t>(q)];
+        }
+
+        bool operator==(const WellBlockAvgPress& that) const
+        {
+            return this->wbp_ == that.wbp_;
+        }
+
+        template <class MessageBufferType>
+        void write(MessageBufferType& buffer) const;
+
+        template <class MessageBufferType>
+        void read(MessageBufferType& buffer);
+
+        template <class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(this->wbp_);
+        }
+
+        static WellBlockAvgPress serializationTestObject()
+        {
+            auto wbp = WellBlockAvgPress{};
+
+            wbp[Quantity::WBP]  = 17.29;
+            wbp[Quantity::WBP4] =  2.718;
+            wbp[Quantity::WBP5] =  3.1415;
+            wbp[Quantity::WBP9] =  1.618;
+
+            return wbp;
+        }
+
+    private:
+        static constexpr auto NumQuantities =
+            static_cast<std::size_t>(Quantity::WBP9) + 1;
+
+        std::array<double, NumQuantities> wbp_{};
+    };
+
+    struct WellFiltrate {
+        double rate{0.};
+        double total{0.};
+        double concentration{0.};
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer) {
+            serializer(rate);
+            serializer(total);
+            serializer(concentration);
+        }
+
+        bool operator==(const WellFiltrate& filtrate) const {
+           return this->rate == filtrate.rate
+              && this->total == filtrate.total
+              && this->concentration == filtrate.concentration;
+        }
+
+        static WellFiltrate serializationTestObject() {
+            WellFiltrate res;
+            res.rate = 1.;
+            res.total = 10.;
+            res.concentration = 0.;
+            return res;
+        }
+
+        template <class MessageBufferType>
+        void write(MessageBufferType& buffer) const;
+
+        template <class MessageBufferType>
+        void read(MessageBufferType& buffer);
+    };
+
     struct Well {
         Rates rates{};
         double bhp{0.0};
@@ -556,6 +760,8 @@ namespace Opm {
         double temperature{0.0};
         int control{0};
 
+        WellFiltrate filtrate;
+
         ::Opm::WellStatus dynamicStatus { Opm::WellStatus::OPEN };
 
         std::vector< Connection > connections{};
@@ -597,16 +803,18 @@ namespace Opm {
 
         bool operator==(const Well& well2) const
         {
-            return rates == well2.rates &&
-                   bhp == well2.bhp &&
-                   thp == well2.thp &&
-                   temperature == well2.temperature &&
-                   control == well2.control &&
-                   dynamicStatus == well2.dynamicStatus &&
-                   connections == well2.connections &&
-                   segments == well2.segments &&
-                   current_control == well2.current_control &&
-                   guide_rates == well2.guide_rates;
+            return (this->rates == well2.rates)
+                && (this->bhp == well2.bhp)
+                && (this->thp == well2.thp)
+                && (this->temperature == well2.temperature)
+                && (this->filtrate == well2.filtrate)
+                && (this->control == well2.control)
+                && (this->dynamicStatus == well2.dynamicStatus)
+                && (this->connections == well2.connections)
+                && (this->segments == well2.segments)
+                && (this->current_control == well2.current_control)
+                && (this->guide_rates == well2.guide_rates)
+                ;
         }
 
         template<class Serializer>
@@ -617,6 +825,7 @@ namespace Opm {
             serializer(thp);
             serializer(temperature);
             serializer(control);
+            serializer(filtrate);
             serializer(dynamicStatus);
             serializer(connections);
             serializer(segments);
@@ -626,21 +835,22 @@ namespace Opm {
 
         static Well serializationTestObject()
         {
-            return Well{Rates::serializationTestObject(),
-                        1.0,
-                        2.0,
-                        3.0,
-                        4,
-                        ::Opm::WellStatus::SHUT,
-                        {Connection::serializationTestObject()},
-                        {{0, Segment::serializationTestObject()}},
-                        CurrentControl::serializationTestObject(),
-                        GuideRateValue::serializationTestObject()
-                   };
+            return Well {
+                Rates::serializationTestObject(),
+                1.0,
+                2.0,
+                3.0,
+                4,
+                WellFiltrate::serializationTestObject(),
+                ::Opm::WellStatus::SHUT,
+                {Connection::serializationTestObject()},
+                {{0, Segment::serializationTestObject()}},
+                CurrentControl::serializationTestObject(),
+                GuideRateValue::serializationTestObject()
+            };
         }
     };
 
-
     class Wells: public std::map<std::string , Well> {
     public:
 
@@ -728,6 +938,34 @@ namespace Opm {
         }
     };
 
+    struct WellBlockAveragePressures
+    {
+        std::unordered_map<std::string, WellBlockAvgPress> values{};
+
+        template <class MessageBufferType>
+        void write(MessageBufferType& buffer) const;
+
+        template <class MessageBufferType>
+        void read(MessageBufferType& buffer);
+
+        bool operator==(const WellBlockAveragePressures& that) const
+        {
+            return this->values == that.values;
+        }
+
+        template <class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(this->values);
+        }
+
+        static WellBlockAveragePressures serializationTestObject()
+        {
+            return {
+                { { "I-45", WellBlockAvgPress::serializationTestObject() } },
+            };
+        }
+    };
 
     /* IMPLEMENTATIONS */
 
@@ -928,6 +1166,18 @@ namespace Opm {
             buffer.write(this->vaporized_water);
     }
 
+    template <class MessageBufferType>
+    void ConnectionFiltrate::write(MessageBufferType& buffer) const {
+        buffer.write(this->rate);
+        buffer.write(this->total);
+        buffer.write(this->skin_factor);
+        buffer.write(this->thickness);
+        buffer.write(this->perm);
+        buffer.write(this->poro);
+        buffer.write(this->radius);
+        buffer.write(this->area_of_flow);
+    }
+
     template <class MessageBufferType>
     void Connection::write(MessageBufferType& buffer) const {
             buffer.write(this->index);
@@ -939,6 +1189,9 @@ namespace Opm {
             buffer.write(this->cell_saturation_gas);
             buffer.write(this->effective_Kh);
             buffer.write(this->trans_factor);
+            buffer.write(this->d_factor);
+            buffer.write(this->compact_mult);
+            this->filtrate.write(buffer);
     }
 
     void Connection::init_json(Json::JsonObject& json_data) const {
@@ -953,6 +1206,8 @@ namespace Opm {
         json_data.add_item("sgas", this->cell_saturation_gas);
         json_data.add_item("Kh", this->effective_Kh);
         json_data.add_item("trans_factor", this->trans_factor);
+        json_data.add_item("d_factor", this->d_factor);
+        json_data.add_item("compact_mult", this->compact_mult);
     }
 
     template <class MessageBufferType>
@@ -964,6 +1219,7 @@ namespace Opm {
         this->velocity.write(buffer);
         this->holdup.write(buffer);
         this->viscosity.write(buffer);
+        this->density.write(buffer);
     }
 
     template <class MessageBufferType>
@@ -978,6 +1234,22 @@ namespace Opm {
         }
     }
 
+    template <class MessageBufferType>
+    void WellBlockAvgPress::write(MessageBufferType& buffer) const
+    {
+        for (const auto& quantity : this->wbp_) {
+            buffer.write(quantity);
+        }
+    }
+
+    template <class MessageBufferType>
+    void WellFiltrate::write(MessageBufferType& buffer) const
+    {
+        buffer.write(this->rate);
+        buffer.write(this->total);
+        buffer.write(this->concentration);
+    }
+
     template <class MessageBufferType>
     void Well::write(MessageBufferType& buffer) const {
         this->rates.write(buffer);
@@ -985,6 +1257,7 @@ namespace Opm {
         buffer.write(this->thp);
         buffer.write(this->temperature);
         buffer.write(this->control);
+        this->filtrate.write(buffer);
 
         {
             const auto status = ::Opm::WellStatus2String(this->dynamicStatus);
@@ -1010,6 +1283,17 @@ namespace Opm {
         this->guide_rates.write(buffer);
     }
 
+    template <class MessageBufferType>
+    void WellBlockAveragePressures::write(MessageBufferType& buffer) const
+    {
+        buffer.write(this->values.size());
+
+        for (const auto& [well, value] : this->values) {
+            buffer.write(well);
+            value.write(buffer);
+        }
+    }
+
     template <class MessageBufferType>
     void Rates::read(MessageBufferType& buffer) {
             buffer.read(this->mask);
@@ -1046,6 +1330,18 @@ namespace Opm {
             buffer.read(this->vaporized_water);
     }
 
+    template <class MessageBufferType>
+    void ConnectionFiltrate::read(MessageBufferType& buffer) {
+        buffer.read(this->rate);
+        buffer.read(this->total);
+        buffer.read(this->skin_factor);
+        buffer.read(this->thickness);
+        buffer.read(this->perm);
+        buffer.read(this->poro);
+        buffer.read(this->radius);
+        buffer.read(this->area_of_flow);
+    }
+
    template <class MessageBufferType>
    void Connection::read(MessageBufferType& buffer) {
             buffer.read(this->index);
@@ -1057,6 +1353,9 @@ namespace Opm {
             buffer.read(this->cell_saturation_gas);
             buffer.read(this->effective_Kh);
             buffer.read(this->trans_factor);
+            buffer.read(this->d_factor);
+            buffer.read(this->compact_mult);
+            this->filtrate.read(buffer);
    }
 
     template <class MessageBufferType>
@@ -1068,6 +1367,7 @@ namespace Opm {
         this->velocity.read(buffer);
         this->holdup.read(buffer);
         this->viscosity.read(buffer);
+        this->density.read(buffer);
     }
 
     template <class MessageBufferType>
@@ -1082,6 +1382,22 @@ namespace Opm {
         }
     }
 
+    template <class MessageBufferType>
+    void WellBlockAvgPress::read(MessageBufferType& buffer)
+    {
+        for (auto& quantity : this->wbp_) {
+            buffer.read(quantity);
+        }
+    }
+
+    template <class MessageBufferType>
+    void WellFiltrate::read(MessageBufferType& buffer)
+    {
+        buffer.read(this->rate);
+        buffer.read(this->total);
+        buffer.read(this->concentration);
+    }
+
     template <class MessageBufferType>
     void Well::read(MessageBufferType& buffer) {
         this->rates.read(buffer);
@@ -1089,6 +1405,7 @@ namespace Opm {
         buffer.read(this->thp);
         buffer.read(this->temperature);
         buffer.read(this->control);
+        this->filtrate.read(buffer);
 
         {
             auto status = std::string{};
@@ -1127,6 +1444,25 @@ namespace Opm {
         this->guide_rates.read(buffer);
     }
 
+    template <class MessageBufferType>
+    void WellBlockAveragePressures::read(MessageBufferType& buffer)
+    {
+        const auto numWells = [&buffer, this]()
+        {
+            auto size = 0*this->values.size();
+            buffer.read(size);
+
+            return size;
+        }();
+
+        auto wellName = std::string{};
+        for (auto well = 0*numWells; well < numWells; ++well) {
+            buffer.read(wellName);
+
+            this->values[wellName].read(buffer);
+        }
+    }
+
     void Well::init_json(Json::JsonObject& json_data) const {
         auto json_connections = json_data.add_array("connections");
         for (const auto& conn : this->connections) {
