@@ -1840,6 +1840,9 @@ START
 DIMENS
   10 10 3 /
 
+NETWORK
+ 3 2 /
+
 GRID
 
 DXV
