@@ -47,6 +47,8 @@
 #include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/common/utility/TimeService.hpp>
 
+#include <opm/input/eclipse/Parser/ParserKeywords/F.hpp>
+
 using namespace Opm;
 
 namespace {
@@ -589,7 +591,9 @@ namespace {
             auto deck = parser.parseString(input);
             const auto& record = deck["WCONHIST"].back().getRecord(0);
             Opm::Well::WellProductionProperties hist(unit_system, "W");
-            hist.handleWCONHIST(alq_type, unit_system, record);
+            hist.handleWCONHIST(alq_type,
+                                Opm::ParserKeywords::FBHPDEF::TARGET_BHP::defaultValue * unit::barsa,
+                                unit_system, record);
 
 
             return hist;
@@ -642,7 +646,9 @@ namespace {
             const auto& kwd     = deck["WCONPROD"].back();
             const auto&  record = kwd.getRecord(0);
             Opm::Well::WellProductionProperties pred(unit_system, "W");
-            pred.handleWCONPROD(alq_type, unit_system, "WELL", record);
+            pred.handleWCONPROD(alq_type,
+                                Opm::ParserKeywords::FBHPDEF::TARGET_BHP::defaultValue * unit::barsa,
+                                unit_system, "WELL", record, {});
 
             return pred;
         }
@@ -1235,7 +1241,7 @@ DATES             -- 2
 /
 
 WCONPROD
- 'P' 'OPEN' 'BHP' 1 2 3 2* 20. 10. 0 13 /
+ 'P' 'OPEN' 'BHP' 1 2 3 2* 20. 2* 13 /
 /
 
 WCONINJE
@@ -1515,10 +1521,8 @@ COMPDAT
 TSTEP
   1 /
 
-
-
 WELSPECS
-  'W1' 'G' 1 1 2005 'LIQ' /
+  'W1' 'G' 1 1 1995.0 'LIQ' /
 /
 
 -- W2
@@ -1538,7 +1542,6 @@ WPAVEDEP
 TSTEP
 1  /
 
-
 COMPDAT
      'W1'     1    1     1    1      'OPEN'  1*     25.620      0.216   2086.842  2*         'Y'      8.486 /
 /
@@ -1547,7 +1550,7 @@ TSTEP
 1 /
 
 WELSPECS
-  'W1' 'G' 1 1 1* 'LIQ' /
+  'W1' 'G' 1 1 -1.0 'LIQ' /
 /
 -- W5
 
@@ -1565,15 +1568,14 @@ END
     const auto& w4 = sched.getWell("W1", 4);
     const auto& w5 = sched.getWell("W1", 5);
 
-
-    BOOST_CHECK_EQUAL(w0.getRefDepth(), grid.getCellDepth(0,0,2));
-    BOOST_CHECK_EQUAL(w0.getRefDepth(), w0.getWPaveRefDepth());
-    BOOST_CHECK_EQUAL(w1.getRefDepth(), w0.getRefDepth());
-    BOOST_CHECK_EQUAL(w2.getRefDepth(), 2005 );
-    BOOST_CHECK_EQUAL(w3.getRefDepth(), grid.getCellDepth(0,0,1));
-    BOOST_CHECK_EQUAL(w4.getRefDepth(), w3.getRefDepth());
-    BOOST_CHECK_EQUAL(w5.getRefDepth(), grid.getCellDepth(0,0,0));
-    BOOST_CHECK_EQUAL(w5.getWPaveRefDepth(), 0);
+    BOOST_CHECK_CLOSE(w0.getRefDepth()     , grid.getCellDepth(0, 0, 2), 1.0e-8);
+    BOOST_CHECK_CLOSE(w0.getRefDepth()     , w0.getWPaveRefDepth()     , 1.0e-8);
+    BOOST_CHECK_CLOSE(w1.getRefDepth()     , w0.getRefDepth()          , 1.0e-8);
+    BOOST_CHECK_CLOSE(w2.getRefDepth()     , 1995.0                    , 1.0e-8);
+    BOOST_CHECK_CLOSE(w3.getRefDepth()     , 1995.0                    , 1.0e-8);
+    BOOST_CHECK_CLOSE(w4.getRefDepth()     , w3.getRefDepth()          , 1.0e-8);
+    BOOST_CHECK_CLOSE(w5.getRefDepth()     , grid.getCellDepth(0, 0, 0), 1.0e-8);
+    BOOST_CHECK_CLOSE(w5.getWPaveRefDepth(), 0.0                       , 1.0e-8);
 }
 
 BOOST_AUTO_TEST_CASE(Missing_RefDepth) {
@@ -1648,3 +1650,317 @@ END
                         R"(Well "W1" must have a BHP reference depth at report=2)");
     BOOST_CHECK_CLOSE(w1.getRefDepth(), es.getInputGrid().getCellDepth(0, 0, 1), 1.0e-8);
 }
+
+BOOST_AUTO_TEST_CASE(Update_Group_Single_Well)
+{
+    const auto deck = Parser{}.parseString(R"(RUNSPEC
+DIMENS
+10 10 3 /
+GRID
+DXV
+10*100.0 /
+DYV
+10*100.0 /
+DZV
+3*5.0 /
+DEPTHZ
+121*2000 /
+PERMX
+300*100.0 /
+COPY
+PERMX PERMY /
+PERMX PERMZ /
+/
+MULTIPLY
+PERMZ 0.1 /
+/
+PORO
+300*0.3 /
+SCHEDULE
+WELSPECS
+ 'P' 'G' 10 10 1* 'OIL' /
+ 'I' 'G'  1  1 1* 'GAS' /
+/
+COMPDAT
+ 'P' 10 10 1 3 'OPEN' /
+ 'I'  1  1 1 1 'OPEN' /
+/
+WCONPROD
+ 'P' 'OPEN' 'LRAT' 1* 1* 1* 1234.567 1* 12.34 /
+/
+WCONINJE
+ 'I' 'GAS' 'OPEN' 'RATE' 20.0E3 /
+/
+TSTEP
+30.0 /
+WELSPECS
+ 'P' 'G1' /
+/
+TSTEP
+ 30.0 /
+END
+)");
+
+    const auto es    = EclipseState { deck };
+    const auto sched = Schedule { deck, es };
+
+    {
+        const auto& wellP = sched.getWell("P", 0);
+        BOOST_CHECK_EQUAL(wellP.groupName(), "G");
+    }
+
+    {
+        const auto& wellP = sched.getWell("P", 1);
+        BOOST_CHECK_EQUAL(wellP.groupName(), "G1");
+    }
+}
+
+BOOST_AUTO_TEST_CASE(Update_Group_Multi_Well)
+{
+    const auto deck = Parser{}.parseString(R"(RUNSPEC
+DIMENS
+10 10 3 /
+GRID
+DXV
+10*100.0 /
+DYV
+10*100.0 /
+DZV
+3*5.0 /
+DEPTHZ
+121*2000 /
+PERMX
+300*100.0 /
+COPY
+PERMX PERMY /
+PERMX PERMZ /
+/
+MULTIPLY
+PERMZ 0.1 /
+/
+PORO
+300*0.3 /
+SCHEDULE
+WELSPECS
+ 'P1' 'G'  1 10 1* 'OIL' /
+ 'P2' 'G' 10  1 1* 'OIL' /
+ 'P3' 'G' 10 10 1* 'OIL' /
+ 'I'  'G'  1  1 1* 'GAS' /
+/
+COMPDAT
+ 'P1'  1 10 1 3 'OPEN' /
+ 'P2' 10  1 1 3 'OPEN' /
+ 'P3' 10 10 1 3 'OPEN' /
+ 'I'   1  1 1 1 'OPEN' /
+/
+WCONPROD
+ 'P*' 'OPEN' 'LRAT' 1* 1* 1* 1234.567 1* 12.34 /
+/
+WCONINJE
+ 'I' 'GAS' 'OPEN' 'RATE' 20.0E3 /
+/
+TSTEP
+30.0 /
+WELSPECS
+ 'P*' 'G1' /
+/
+TSTEP
+ 30.0 /
+END
+)");
+
+    const auto es    = EclipseState { deck };
+    const auto sched = Schedule { deck, es };
+
+    {
+        for (const auto* P : { "P1", "P2", "P3", }) {
+            const auto& wellP = sched.getWell(P, 0);
+            BOOST_CHECK_MESSAGE(wellP.groupName() == "G",
+                                "Well " << P << " must have "
+                                "controlling group \"G\" at time zero");
+        }
+    }
+
+    {
+        for (const auto* P : { "P1", "P2", "P3", }) {
+            const auto& wellP = sched.getWell(P, 1);
+            BOOST_CHECK_MESSAGE(wellP.groupName() == "G1",
+                                "Well " << P << " must have "
+                                "controlling group \"G1\" at time one");
+        }
+    }
+}
+
+BOOST_AUTO_TEST_CASE(Update_Group_WList)
+{
+    const auto deck = Parser{}.parseString(R"(RUNSPEC
+DIMENS
+10 10 3 /
+GRID
+DXV
+10*100.0 /
+DYV
+10*100.0 /
+DZV
+3*5.0 /
+DEPTHZ
+121*2000 /
+PERMX
+300*100.0 /
+COPY
+PERMX PERMY /
+PERMX PERMZ /
+/
+MULTIPLY
+PERMZ 0.1 /
+/
+PORO
+300*0.3 /
+SCHEDULE
+WELSPECS
+ 'P1' 'G'  1 10 1* 'OIL' /
+ 'P2' 'G' 10  1 1* 'OIL' /
+ 'P3' 'G' 10 10 1* 'OIL' /
+ 'I'  'G'  1  1 1* 'GAS' /
+/
+COMPDAT
+ 'P1'  1 10 1 3 'OPEN' /
+ 'P2' 10  1 1 3 'OPEN' /
+ 'P3' 10 10 1 3 'OPEN' /
+ 'I'   1  1 1 1 'OPEN' /
+/
+WLIST
+ '*QFS' NEW 'I' 'P3' /
+/
+WCONPROD
+ 'P*' 'OPEN' 'LRAT' 1* 1* 1* 1234.567 1* 12.34 /
+/
+WCONINJE
+ 'I' 'GAS' 'OPEN' 'RATE' 20.0E3 /
+/
+TSTEP
+30.0 /
+WELSPECS
+ '*QFS' 'G1' /
+/
+TSTEP
+ 30.0 /
+END
+)");
+
+    const auto es    = EclipseState { deck };
+    const auto sched = Schedule { deck, es };
+
+    {
+        for (const auto* P : { "P1", "P2", "P3", "I", }) {
+            const auto& wellP = sched.getWell(P, 0);
+            BOOST_CHECK_MESSAGE(wellP.groupName() == "G",
+                                "Well " << P << " must have "
+                                "controlling group \"G\" at time zero");
+        }
+    }
+
+    {
+        for (const auto* P : { "P1", "P2", }) {
+            const auto& wellP = sched.getWell(P, 1);
+            BOOST_CHECK_MESSAGE(wellP.groupName() == "G",
+                                "Well " << P << " must have "
+                                "controlling group \"G\" at time one");
+        }
+
+        for (const auto* P : { "P3", "I", }) {
+            const auto& wellP = sched.getWell(P, 1);
+            BOOST_CHECK_MESSAGE(wellP.groupName() == "G1",
+                                "Well " << P << " must have "
+                                "controlling group \"G1\" at time one");
+        }
+    }
+}
+
+BOOST_AUTO_TEST_CASE(FBHPDEF_Basic)
+{
+    const auto deck = Parser{}.parseString(R"(RUNSPEC
+DIMENS
+10 10 3 /
+GRID
+DXV
+10*100.0 /
+DYV
+10*100.0 /
+DZV
+3*5.0 /
+DEPTHZ
+121*2000 /
+PERMX
+300*100.0 /
+COPY
+PERMX PERMY /
+PERMX PERMZ /
+/
+MULTIPLY
+PERMZ 0.1 /
+/
+PORO
+300*0.3 /
+SCHEDULE
+
+WELSPECS
+ 'P' 'G' 10 10 1* 'OIL' /
+ 'I' 'G'  1  1 1* 'GAS' /
+ 'I2' 'W'  1  1 1* 'WATER' /
+ 'I3' 'W'  1  1 1* 'WATER' /
+/
+COMPDAT
+ 'P' 10 10 1 3 'OPEN' /
+ 'I'  1  1 1 1 'OPEN' /
+ 'I2'  1  1 1 1 'OPEN' /
+ 'I3'  1  1 1 1 'OPEN' /
+/
+
+WCONINJH
+  I3 WATER OPEN 116281 1* 0 /
+/
+
+FBHPDEF
+  5.0 20.0 /
+
+WCONPROD
+ 'P' 'OPEN' 'LRAT' 1* 1* 1* 1234.567 1* 1* /
+/
+
+WCONINJH
+  I2 WATER OPEN 116281 1* 0 /
+/
+
+FBHPDEF
+  2.0 30.0 /
+
+WCONINJE
+ 'I' 'GAS' 'OPEN' 'RATE' 20.0E3 /
+/
+
+TSTEP
+30.0 /
+WELSPECS
+ 'P' 'G1' /
+/
+TSTEP
+ 30.0 /
+END
+)");
+
+    const auto es    = EclipseState { deck };
+    const auto sched = Schedule { deck, es };
+
+    const auto& wellP = sched.getWell("P", 0);
+    BOOST_CHECK_EQUAL(wellP.getProductionProperties().BHPTarget.get<double>(), 5.0);
+
+    const auto& wellI = sched.getWell("I", 0);
+    BOOST_CHECK_CLOSE(wellI.getInjectionProperties().BHPTarget.get<double>(), 30.0, 1e-12);
+
+    const auto& wellI2 = sched.getWell("I2", 0);
+    BOOST_CHECK_EQUAL(wellI2.getInjectionProperties().bhp_hist_limit, 20.0 * unit::barsa);
+
+    const auto& wellI3 = sched.getWell("I3", 0);
+    BOOST_CHECK_CLOSE(wellI3.getInjectionProperties().bhp_hist_limit, 6891.2 * unit::barsa, 1e-12);
+}
