@@ -35,8 +35,8 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
-#include <utility>
 #include <type_traits>
+#include <utility>
 #include <vector>
 
 // ===========================================================================
@@ -57,12 +57,22 @@ namespace {
         const auto j = (dims[1] - 1) - 1;
 
         for (auto k = top; k < static_cast<int>(dims.size()); ++k) {
+            const auto depth = 2000 + (2*k + 1) / static_cast<double>(2);
+
+            auto ctf_props = Opm::Connection::CTFProperties{};
+            ctf_props.CF = k / 100.0;
+            ctf_props.Kh = 1.0;
+            ctf_props.Ke = 1.0;
+            ctf_props.rw = 1.0;
+            ctf_props.r0 = 0.5;
+            ctf_props.re = 0.5;
+            ctf_props.connection_length = 1.0;
+
             conns.emplace_back(i, j, k, globIndex({i, j, k}, dims), k,
-                               2000 + (2*k + 1) / static_cast<double>(2),
                                Opm::Connection::State::OPEN,
-                               k / 100.0, 1.0, 1.0, 0.5, 0.5, 1.0, 0.0, 0,
                                Opm::Connection::Direction::Z,
-                               Opm::Connection::CTFKind::DeckValue, k, false);
+                               Opm::Connection::CTFKind::DeckValue,
+                               0, depth, ctf_props, k, false);
         }
 
         return { Opm::Connection::Order::INPUT, i, j, conns };
@@ -87,18 +97,28 @@ namespace {
         };
 
         for (auto k = topConn; k < kMax; ++k) {
+            const auto depth = 2000 + (2*k + 1) / static_cast<double>(2);
+
+            auto ctf_props = Opm::Connection::CTFProperties{};
+
+            // 0.03, 0.0, 0.01, 0.02, 0.03, ...
+            ctf_props.CF = ((k + 3 - topConn) % 4) / 100.0;
+
+            ctf_props.Kh = 1.0;
+            ctf_props.Ke = 1.0;
+            ctf_props.rw = 1.0;
+            ctf_props.r0 = 0.5;
+            ctf_props.re = 0.5;
+            ctf_props.connection_length = 1.0;
+
             conns.emplace_back(i, j, k, globIndex({i, j, k}, dims), k - topConn,
-                               2000 + (2*k + 1) / static_cast<double>(2),
 
                                // Open, Shut, Open, Open, Shut, ...
                                state[(k - topConn) % state.size()],
 
-                               // 0.03, 0.0, 0.01, 0.02, 0.03, ...
-                               ((k + 3 - topConn) % 4) / 100.0,
-
-                               1.0, 1.0, 0.5, 0.5, 1.0, 0.0, 0,
                                Opm::Connection::Direction::Z,
-                               Opm::Connection::CTFKind::DeckValue, k - topConn, false);
+                               Opm::Connection::CTFKind::DeckValue,
+                               0, depth, ctf_props, k - topConn, false);
         }
 
         return { Opm::Connection::Order::INPUT, i, j, conns };
@@ -115,12 +135,22 @@ namespace {
         const auto iMax = std::min(dims[0] - 1, left + numConns);
 
         for (auto i = left; i < iMax; ++i) {
+            const auto depth = 2000 + (2*k + 1) / static_cast<double>(2);
+
+            auto ctf_props = Opm::Connection::CTFProperties{};
+            ctf_props.CF = i / 100.0;
+            ctf_props.Kh = 1.0;
+            ctf_props.Ke = 1.0;
+            ctf_props.rw = 1.0;
+            ctf_props.r0 = 0.5;
+            ctf_props.re = 0.5;
+            ctf_props.connection_length = 1.0;
+
             conns.emplace_back(i, j, k, globIndex({i, j, k}, dims), i - left,
-                               2000 + (2*k + 1) / static_cast<double>(2),
                                Opm::Connection::State::OPEN,
-                               i / 100.0, 1.0, 1.0, 0.5, 0.5, 1.0, 0.0, 0,
                                Opm::Connection::Direction::X,
-                               Opm::Connection::CTFKind::DeckValue, i - left, false);
+                               Opm::Connection::CTFKind::DeckValue,
+                               0, depth, ctf_props, i - left, false);
         }
 
         return { Opm::Connection::Order::INPUT, left, j, conns };
@@ -147,6 +177,21 @@ namespace {
         std::sort(v.begin(), v.end());
         return v;
     }
+
+    template <typename T>
+    std::vector<bool>
+    makeActiveMap(const std::vector<T>& v, const std::vector<T>& activeElements)
+    {
+        auto isActive = std::vector<bool>{};
+        isActive.reserve(v.size());
+
+        for (const auto& elm : v) {
+            isActive.push_back(std::binary_search(activeElements.begin(),
+                                                  activeElements.end(), elm));
+        }
+
+        return isActive;
+    }
 } // Anonymous namespace
 
 BOOST_AUTO_TEST_SUITE(Basic_Operations)
@@ -263,6 +308,88 @@ BOOST_AUTO_TEST_CASE(Construct_Horizontal_X_5_Cols)
                                   expectConns.begin(), expectConns.end());
 }
 
+BOOST_AUTO_TEST_CASE(Prune_Inactive_Cells)
+{
+    const auto dims = std::array { 10, 10, 3 };
+
+    // Producer connected in Z direction in column (9,9) of all layers,
+    // meaning cells (9,9,1), (9,9,2), and (9,9,3).
+    auto prod = Opm::PAvgCalculator {
+        shoeBox(dims), qfsProducer(dims)
+    };
+
+    const auto expectCells = sort(std::vector {
+        globIndex({7, 7, 0}, dims),                             globIndex({9, 7, 0}, dims),
+                                    globIndex({8, 8, 0}, dims),
+        globIndex({7, 9, 0}, dims),                             globIndex({9, 9, 0}, dims),
+
+                                    globIndex({8, 7, 1}, dims),
+        globIndex({7, 8, 1}, dims), globIndex({8, 8, 1}, dims), globIndex({9, 8, 1}, dims),
+                                    globIndex({8, 9, 1}, dims),
+
+        globIndex({7, 7, 2}, dims), globIndex({8, 7, 2}, dims), globIndex({9, 7, 2}, dims),
+        globIndex({7, 8, 2}, dims), globIndex({8, 8, 2}, dims), globIndex({9, 8, 2}, dims),
+        globIndex({7, 9, 2}, dims), globIndex({8, 9, 2}, dims), globIndex({9, 9, 2}, dims),
+    });
+
+    prod.pruneInactiveWBPCells(makeActiveMap(prod.allWBPCells(), expectCells));
+
+    const auto wbpCells = sort(prod.allWBPCells());
+    const auto wbpConns = sort(prod.allWellConnections());
+
+    const auto expectConns = std::vector {
+        std::size_t{0}, std::size_t{1}, std::size_t{2},
+    };
+
+    BOOST_CHECK_EQUAL_COLLECTIONS(wbpCells   .begin(), wbpCells   .end(),
+                                  expectCells.begin(), expectCells.end());
+
+    BOOST_CHECK_EQUAL_COLLECTIONS(wbpConns   .begin(), wbpConns   .end(),
+                                  expectConns.begin(), expectConns.end());
+}
+
+BOOST_AUTO_TEST_CASE(Prune_Inactive_Connections)
+{
+    const auto dims = std::array { 10, 10, 3 };
+
+    // Producer connected in Z direction in column (9,9) of all layers,
+    // meaning cells (9,9,1), (9,9,2), and (9,9,3).
+    auto prod = Opm::PAvgCalculator {
+        shoeBox(dims), qfsProducer(dims)
+    };
+
+    const auto expectCells = sort(std::vector {
+        globIndex({7, 7, 0}, dims), globIndex({8, 7, 0}, dims), globIndex({9, 7, 0}, dims),
+        globIndex({7, 8, 0}, dims),                             globIndex({9, 8, 0}, dims),
+        globIndex({7, 9, 0}, dims), globIndex({8, 9, 0}, dims), globIndex({9, 9, 0}, dims),
+
+        globIndex({7, 7, 1}, dims), globIndex({8, 7, 1}, dims), globIndex({9, 7, 1}, dims),
+        globIndex({7, 8, 1}, dims),                             globIndex({9, 8, 1}, dims),
+        globIndex({7, 9, 1}, dims), globIndex({8, 9, 1}, dims), globIndex({9, 9, 1}, dims),
+
+        globIndex({7, 7, 2}, dims), globIndex({8, 7, 2}, dims), globIndex({9, 7, 2}, dims),
+        globIndex({7, 8, 2}, dims),                             globIndex({9, 8, 2}, dims),
+        globIndex({7, 9, 2}, dims), globIndex({8, 9, 2}, dims), globIndex({9, 9, 2}, dims),
+    });
+
+    prod.pruneInactiveWBPCells(makeActiveMap(prod.allWBPCells(), expectCells));
+
+    const auto wbpCells = sort(prod.allWBPCells());
+    const auto wbpConns = sort(prod.allWellConnections());
+
+    const auto expectConns = std::vector {
+        std::size_t{0}, std::size_t{1}, std::size_t{2},
+    };
+
+    BOOST_CHECK_EQUAL_COLLECTIONS(wbpCells   .begin(), wbpCells   .end(),
+                                  expectCells.begin(), expectCells.end());
+
+    // Well connection source locations must be full input set even if all
+    // connections are in deactivated cells.
+    BOOST_CHECK_EQUAL_COLLECTIONS(wbpConns   .begin(), wbpConns   .end(),
+                                  expectConns.begin(), expectConns.end());
+}
+
 BOOST_AUTO_TEST_SUITE_END() // Basic_Operations
 
 // ===========================================================================
