@@ -17,9 +17,8 @@
 */
 #include "config.h"
 
-#include <cstdlib>
+#define BOOST_TEST_MODULE Restart_File_IO
 
-#define BOOST_TEST_MODULE EclipseIO
 #include <boost/test/unit_test.hpp>
 
 #include <opm/output/data/Cells.hpp>
@@ -36,6 +35,8 @@
 
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FIPRegionStatistics.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/RegionSetMatcher.hpp>
 #include <opm/input/eclipse/EclipseState/IOConfig/IOConfig.hpp>
 #include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/Eqldims.hpp>
@@ -54,16 +55,26 @@
 #include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
+
 #include <opm/input/eclipse/Utility/Functional.hpp>
 
-#include <opm/input/eclipse/Parser/Parser.hpp>
-
-#include <opm/input/eclipse/Deck/Deck.hpp>
-
 #include <opm/common/utility/TimeService.hpp>
 
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
+#include <opm/input/eclipse/Parser/Parser.hpp>
+
+#include <cstdlib>
+#include <ctime>
+#include <map>
+#include <memory>
+#include <optional>
 #include <sstream>
+#include <stdexcept>
+#include <string>
+#include <string_view>
 #include <tuple>
+#include <vector>
 
 #include <tests/WorkArea.hpp>
 
@@ -187,15 +198,15 @@ data::Wells mkWells() {
      *  input deck. All other entries in the well structures are arbitrary.
      */
     Opm::data::ConnectionFiltrate con_filtrate {0.1, 1, 3, 0.4, 1.e-9, 0.2, 0.05, 10.}; // values are not used in this test
-    w1.connections.push_back( { 88, rc1, 30.45, 123.4, 543.21, 0.62, 0.15, 1.0e3, 1.234, con_filtrate } );
-    w1.connections.push_back( { 288, rc2, 33.19, 123.4, 432.1, 0.26, 0.45, 2.56, 2.345, con_filtrate } );
+    w1.connections.push_back( { 88, rc1, 30.45, 123.4, 543.21, 0.62, 0.15, 1.0e3, 1.234, 0.0, 1.23, con_filtrate } );
+    w1.connections.push_back( { 288, rc2, 33.19, 123.4, 432.1, 0.26, 0.45, 2.56, 2.345, 0.0, 0.98, con_filtrate } );
 
     w2.rates = r2;
     w2.thp = 2.0;
     w2.bhp = 2.34;
     w2.temperature = 4.56;
     w2.control = 2;
-    w2.connections.push_back( { 188, rc3, 36.22, 123.4, 256.1, 0.55, 0.0125, 314.15, 3.456, con_filtrate } );
+    w2.connections.push_back( { 188, rc3, 36.22, 123.4, 256.1, 0.55, 0.0125, 314.15, 3.456, 0.0, 2.46, con_filtrate } );
 
     {
         data::Wells wellRates;
@@ -218,16 +229,42 @@ data::Solution mkSolution(int numCells)
         { "SGAS",     data::CellData { measure::identity,    {}, data::TargetType::RESTART_SOLUTION } },
     };
 
-    sol.data("PRESSURE").assign( numCells, 6.0 );
-    sol.data("TEMP").assign( numCells, 7.0 );
-    sol.data("SWAT").assign( numCells, 8.0 );
-    sol.data("SGAS").assign( numCells, 9.0 );
+    sol.data<double>("PRESSURE").assign( numCells, 6.0 );
+    sol.data<double>("TEMP").assign( numCells, 7.0 );
+    sol.data<double>("SWAT").assign( numCells, 8.0 );
+    sol.data<double>("SGAS").assign( numCells, 9.0 );
 
-    fun::iota rsi( 300, 300 + numCells );
-    fun::iota rvi( 400, 400 + numCells );
+    fun::iota rsi( 300.0, 300.0 + numCells );
+    fun::iota rvi( 400.0, 400.0 + numCells );
 
-    sol.insert("RS", measure::identity, { rsi.begin(), rsi.end() }, data::TargetType::RESTART_SOLUTION);
-    sol.insert("RV", measure::identity, { rvi.begin(), rvi.end() }, data::TargetType::RESTART_SOLUTION);
+    sol.insert("RS", measure::identity,
+               { rsi.begin(), rsi.end() },
+               data::TargetType::RESTART_SOLUTION);
+    sol.insert("RV", measure::identity,
+               { rvi.begin(), rvi.end() },
+               data::TargetType::RESTART_SOLUTION);
+
+    return sol;
+}
+
+data::Solution mkSolutionFIP(const int numCells)
+{    using measure = UnitSystem::measure;
+
+    auto sol = data::Solution {
+        { "PRESSURE", data::CellData { measure::pressure, {}, data::TargetType::RESTART_SOLUTION } },
+        { "SWAT",     data::CellData { measure::identity, {}, data::TargetType::RESTART_SOLUTION } },
+        { "SGAS",     data::CellData { measure::identity, {}, data::TargetType::RESTART_SOLUTION } },
+        { "FIPOIL",   data::CellData { measure::identity, {}, data::TargetType::RESTART_SOLUTION } },
+        { "FIPWAT",   data::CellData { measure::identity, {}, data::TargetType::RESTART_SOLUTION } },
+        { "FIPGAS",   data::CellData { measure::identity, {}, data::TargetType::RESTART_SOLUTION } },
+    };
+
+    sol.data<double>("PRESSURE").assign(numCells, 6.0);
+    sol.data<double>("SWAT").assign(numCells, 8.0);
+    sol.data<double>("SGAS").assign(numCells, 9.0);
+    sol.data<double>("FIPOIL").assign(numCells, 10.0);
+    sol.data<double>("FIPWAT").assign(numCells, 11.0);
+    sol.data<double>("FIPGAS").assign(numCells, 12.0);
 
     return sol;
 }
@@ -428,11 +465,13 @@ first_sim(const Setup&   setup,
     const auto groups = mkGroups();
     const auto& udq = setup.schedule.getUDQConfig(report_step);
     auto segmentMatcherFactory = []() { return std::make_unique<SegmentMatcher>(ScheduleState{}); };
+    auto regionSetMatcherFactory = []() { return std::make_unique<RegionSetMatcher>(FIPRegionStatistics {}); };
 
     udq.eval(report_step,
              setup.schedule,
              setup.schedule.wellMatcher(report_step),
              segmentMatcherFactory,
+             regionSetMatcherFactory,
              st, udq_state);
 
     RestartValue restart_value(sol, wells, groups, {});
@@ -472,10 +511,10 @@ void compare(const RestartValue&            fst,
             tol *= 10.0;
         }
 
-        auto first = fst.solution.data(key).begin();
-        auto second = snd.solution.data(key).begin();
+        auto first = fst.solution.data<double>(key).begin();
+        auto second = snd.solution.data<double>(key).begin();
 
-        for (; first != fst.solution.data( key).end(); ++first, ++second) {
+        for (; first != fst.solution.data<double>(key).end(); ++first, ++second) {
             BOOST_CHECK_CLOSE(*first, *second, tol);
         }
     }
@@ -608,10 +647,10 @@ void compare_equal(const RestartValue&            fst,
 {
     for (const auto& value : keys) {
         const std::string& key = value.key;
-        auto first = fst.solution.data( key ).begin();
-        auto second = snd.solution.data( key ).begin();
+        auto first = fst.solution.data<double>(key).begin();
+        auto second = snd.solution.data<double>(key).begin();
 
-        for (; first != fst.solution.data(key).end(); ++first, ++second) {
+        for (; first != fst.solution.data<double>(key).end(); ++first, ++second) {
             BOOST_CHECK_EQUAL(*first, *second);
         }
     }
@@ -1290,3 +1329,90 @@ BOOST_AUTO_TEST_CASE(ReadWrite_NumericalAquifer_Data)
 
     BOOST_CHECK_MESSAGE(src == dest, "Serialised/deserialised Numerical aquifer object must be equal to source object");
 }
+
+namespace {
+    void checkAllEntriesAre(const double x,
+                            std::string_view arrayName,
+                            const std::vector<double>& v,
+                            const double tol = 1.0e-8)
+    {
+        auto i = 0;
+        for (const auto& vi : v) {
+            BOOST_TEST_MESSAGE(arrayName << '[' << (i++) << ']');
+            BOOST_CHECK_CLOSE(vi, x, tol);
+        }
+    }
+}
+
+BOOST_AUTO_TEST_CASE(Fluid_In_Place)
+{
+    namespace OS = ::Opm::EclIO::OutputStream;
+
+    WorkArea test_area("test_Restart");
+    test_area.copyIn("BASE_SIM.DATA");
+
+    const Setup base_setup("BASE_SIM.DATA");
+
+    const auto num_cells = base_setup.grid.getNumActive();
+    const auto cells = mkSolutionFIP(num_cells);
+    const auto wells = mkWells();
+    const auto groups = mkGroups();
+    const auto sumState = sim_state(base_setup.schedule);
+    const auto udqState = UDQState{1};
+    auto aquiferData = std::optional<Opm::RestartIO::Helpers::AggregateAquiferData>{std::nullopt};
+    const Action::State action_state{};
+    const WellTestState wtest_state{};
+
+    const RestartValue restart_value(cells, wells, groups, {});
+
+    const auto outputDir = test_area.currentWorkingDirectory();
+
+    {
+        const auto seqnum = 1;
+
+        auto rstFile = OS::Restart {
+            OS::ResultSet{ outputDir, "BASE_SIM" }, seqnum,
+            OS::Formatted{ false }, OS::Unified{ true }
+        };
+
+        RestartIO::save(rstFile, seqnum,
+                        100,
+                        restart_value,
+                        base_setup.es,
+                        base_setup.grid,
+                        base_setup.schedule,
+                        action_state,
+                        wtest_state,
+                        sumState,
+                        udqState,
+                        aquiferData,
+                        true);
+    }
+
+    {
+        const auto rstFile = ::Opm::EclIO::OutputStream::
+            outputFileName({outputDir, "BASE_SIM"}, "UNRST");
+
+        EclIO::ERst rst{ rstFile };
+
+        BOOST_CHECK_MESSAGE(rst.hasKey("FIPOIL"), R"(Restart file must have "FIPOIL" vector)");
+        checkAllEntriesAre(10.0, "FIPOIL", rst.getRestartData<double>("FIPOIL", 1));
+
+        BOOST_CHECK_MESSAGE(rst.hasKey("FIPWAT"), R"(Restart file must have "FIPWAT" vector)");
+        checkAllEntriesAre(11.0, "FIPWAT", rst.getRestartData<double>("FIPWAT", 1));
+
+        BOOST_CHECK_MESSAGE(rst.hasKey("FIPGAS"), R"(Restart file must have "FIPGAS" vector)");
+        checkAllEntriesAre(12.0, "FIPGAS", rst.getRestartData<double>("FIPGAS", 1));
+
+        // SFIP* "alias" arrays must be present when the data::Solution
+        // object "only" provides basic FIP* result arrays.
+        BOOST_CHECK_MESSAGE(rst.hasKey("SFIPOIL"), R"(Restart file must have "SFIPOIL" vector)");
+        checkAllEntriesAre(10.0, "SFIPOIL", rst.getRestartData<double>("SFIPOIL", 1));
+
+        BOOST_CHECK_MESSAGE(rst.hasKey("SFIPWAT"), R"(Restart file must have "SFIPWAT" vector)");
+        checkAllEntriesAre(11.0, "SFIPWAT", rst.getRestartData<double>("SFIPWAT", 1));
+
+        BOOST_CHECK_MESSAGE(rst.hasKey("SFIPGAS"), R"(Restart file must have "SFIPGAS" vector)");
+        checkAllEntriesAre(12.0, "SFIPGAS", rst.getRestartData<double>("SFIPGAS", 1));
+    }
+}
