@@ -40,6 +40,7 @@ BOOST_AUTO_TEST_CASE(Create)
         c.insert("NAME", UnitSystem::measure::identity, data , data::TargetType::RESTART_SOLUTION);
         BOOST_CHECK_EQUAL( c.size() , 1U );
         BOOST_CHECK_EQUAL( c.has("NAME") , true);
+        BOOST_CHECK_THROW(c.data<int>("NAME"), std::bad_variant_access);
 
         BOOST_CHECK_EQUAL( c.find("NAME")->first, "NAME");
 
@@ -77,6 +78,24 @@ BOOST_AUTO_TEST_CASE(Create2)
 }
 
 
+BOOST_AUTO_TEST_CASE(CreateInt)
+{
+    std::vector<int> data(100);
+    const auto c = data::Solution {
+        { "CNV_OIL", data::CellData { data, data::TargetType::RESTART_SOLUTION } },
+        { "CNV_GAS", data::CellData { data, data::TargetType::RESTART_SOLUTION } },
+        { "CNV_WAT", data::CellData { data, data::TargetType::RESTART_SOLUTION } },
+    };
+
+    const auto c2 = c;
+    BOOST_CHECK_EQUAL( c2.size() , 3U );
+    BOOST_CHECK( c2.has("CNV_OIL") );
+    const auto& fld = c2.data<int>("CNV_WAT");
+    BOOST_CHECK_EQUAL_COLLECTIONS(data.begin(), data.end(), fld.begin(), fld.end());
+    BOOST_CHECK_THROW(c2.data<double>("CNV_GAS"), std::bad_variant_access);
+}
+
+
 
 BOOST_AUTO_TEST_CASE(UNITS) {
     std::vector<double> data(100,1);
@@ -85,12 +104,12 @@ BOOST_AUTO_TEST_CASE(UNITS) {
 
     c.insert("NAME", UnitSystem::measure::pressure, data , data::TargetType::RESTART_SOLUTION);
 
-    double si0 = c.data("NAME")[0];
+    double si0 = c.data<double>("NAME")[0];
     c.convertFromSI( metric );
-    double metric0 = c.data("NAME")[0];
+    double metric0 = c.data<double>("NAME")[0];
     c.convertFromSI( metric );
-    BOOST_CHECK_EQUAL( metric0 , c.data("NAME")[0] );
+    BOOST_CHECK_EQUAL( metric0 , c.data<double>("NAME")[0] );
     c.convertToSI( metric );
-    BOOST_CHECK_EQUAL( si0 , c.data("NAME")[0] );
+    BOOST_CHECK_EQUAL( si0 , c.data<double>("NAME")[0] );
 }
 
